
from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.base.base_entity import BaseEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class HistoryCollector(BaseEntity):
    '''This managed object type enables clients to retrieve historical data and
    receive updates when the server appends new data to a collection. This is a
    base type for item-specific types related to event or task history. Historical
    data is inherently append-only, although a server administrator may
    periodically purge old data.Typically, a client creates a history collector by
    using a filter on a potentially large set, such as all the events in a
    datacenter. The collector provides access to the items that match the filter,
    which could also be a relatively large set.The items in a collector are always
    ordered by date and time of creation. Item properties normally include this
    time stamp.The client may set the "viewable latest page" for the collector,
    which is the contiguous subset of the items that are of immediate interest.
    These items are available as the "latestPage" property, which the client may
    retrieve and monitor by using the PropertyCollector managed object.Clients can
    change the page size of the "latestPage" by using setLatestPageSize().The
    client may use the following features of the history collector.*
    RewindCollector - Moves the "scrollable view" to the oldest item (the default
    setting). * readNext - Retrieves all the items in the collector, from the
    oldest item to the newest item. Retrieves either tasks or events depending on
    the operation. * readPrev - Retrieves all items (excluding the "viewable latest
    page") in the collector, from the newest item to the oldest item. Retrieves
    either tasks or events depending on the operation. * ResetCollector - Moves the
    "scrollable view" to the item immediately preceding the "viewable latest page".'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.HistoryCollector):
        super(HistoryCollector, self).__init__(core, name=name, ref=ref, type=type)

    
    @property
    def filter(self):
        '''The filter used to create this collector.'''
        return self.update('filter')

    
    
    def DestroyCollector(self):
        '''Destroys this collector.
        
        '''
        return self.delegate("DestroyCollector")()
    
    def ResetCollector(self):
        '''Moves the "scrollable view" to the item immediately preceding the "viewable
        latest page". If you use "readPrev", ReadPreviousTasks or ReadPreviousEvents,
        all items are retrieved from the newest item to the oldest item.
        
        '''
        return self.delegate("ResetCollector")()
    
    def RewindCollector(self):
        '''Moves the "scrollable view" to the oldest item. If you use ReadNextTasks or
        ReadNextEvents, all items are retrieved from the oldest item to the newest
        item. This is the default setting when the collector is created.
        
        '''
        return self.delegate("RewindCollector")()
    
    def SetCollectorPageSize(self, maxCount):
        '''Sets the "viewable latest page" size to contain at most the number of items
        specified by the maxCount parameter).
        
        :param maxCount: The maximum number of items in the page.
        
        '''
        return self.delegate("SetCollectorPageSize")(maxCount)



from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class VmProcess(Base):
    '''
    Operations on running virtual machine processes
    '''
    moid = 'ha-cli-handler-vm-process'
    def list(self):
        '''
        List the virtual machines on this system. This command currently will only list running VMs on the system.
        :returns: vim.EsxCLI.vm.process.list.VirtualMachine[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.vm.process.List',
                            )
    def kill(self, type, worldid):
        '''
        Used to forcibly kill Virtual Machines that are stuck and not responding to normal stop operations.
        :param type: string, The type of kill operation to attempt. There are three types of VM kills that can be attempted:   [soft, hard, force]. Users should always attempt 'soft' kills first, which will give the VMX process a chance to shutdown cleanly (like kill or kill -SIGTERM). If that does not work move to 'hard' kills which will shutdown the process immediately (like kill -9 or kill -SIGKILL). 'force' should be used as a last resort attempt to kill the VM. If all three fail then a reboot is required.
        :param worldid: long, The World ID of the Virtual Machine to kill. This can be obtained from the 'vm process list' command
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.vm.process.Kill',
                            type=type,
                            worldid=worldid,
                            )   



########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

VirtualMachineFlagInfoVirtualMmuUsage = Enum(
    'automatic',
    'off',
    'on',

)




from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.base.base_entity import BaseEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class AuthorizationManager(BaseEntity):
    '''This managed object provides operations to query and update roles and
    permissions.are the basic individual rights required to perform operations.
    They are statically defined and never change for a single version of a product.
    Examples of privileges are "Power on a virtual machine" or "Configure a
    host."are aggregations of privileges, used for convenience. For user-defined
    roles, the system-defined privileges, "System.Anonymous", "System.View", and
    "System.Read" are always present.are the actual access-control rules. A
    permission is defined on a ManagedEntity and specifies the user or group
    ("principal") to which the rule applies. The role specifies the privileges to
    apply, and the propagate flag specifies whether or not the rule applies to sub-
    objects of the managed entity.A ManagedEntity may have multiple permissions,
    but may have only one permission per user or group. If, when logging in, a user
    has both a user permission and a group permission (as a group member) for the
    same entity, then the user-specific permission takes precedent. If there is no
    user-specific permission, but two or more group permissions are present, and
    the user is a member of the groups, then the privileges are the union of the
    specified roles.Managed entities may be collected together into a "complex
    entity" for the purpose of applying permissions consistently. Complex entities
    may have a Datacenter, ComputeResource, or ClusterComputeResource as a parent,
    with other child managed objects as additional parts of the complex entity:* A
    Datacenter's child objects are the root virtual machine and host Folders. * A
    ComputeResource's child objects are the root ResourcePool and HostSystem. * A
    ClusterComputeResource has only the root ResourcePool as a child object.Child
    objects in a complex entity are forced to inherit permissions from the parent
    object. When query operations are used to discover permissions on child objects
    of complex entities, different results may be returned for the owner of the
    permission. In some cases, the child object of the complex entity is returned
    as the object that defines the permission, and in other cases, the parent from
    which the permission is propagated is returned as the object that defines the
    permission. In both cases, the information about the owner of the permission is
    correct, since the entities within a complex entity are considered equivalent.
    Permissions defined on complex entities are always applicable on the child
    entities, regardless of the propagation flag, but may only be defined or
    modified on the parent object.In a group of fault-tolerance (FT) protected
    VirtualMachines, the secondary VirtualMachines are forced to inherit
    permissions from the primary VirtualMachine. Queries to discover permissions on
    FT secondary VMs always return the primary VM as the object that defines the
    permissions. Permissions defined on an FT primary VM are always applicable on
    its secondary VMs, but can only be defined or modified on the primary VM.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.AuthorizationManager):
        super(AuthorizationManager, self).__init__(core, name=name, ref=ref, type=type)

    
    @property
    def description(self):
        '''Static, descriptive strings for system roles and privileges.'''
        return self.update('description')
    @property
    def privilegeList(self):
        '''The list of system-defined privileges.'''
        return self.update('privilegeList')
    @property
    def roleList(self):
        '''The currently defined roles in the system, including static system-defined
        roles.'''
        return self.update('roleList')

    
    
    def AddAuthorizationRole(self, name, privIds=None):
        '''Adds a new role. This method will add a user-defined role with given list of
        privileges and three system-defined privileges, "System.Anonymous",
        "System.View", and "System.Read".
        
        :param name: Name of the new role.
        
        :param privIds: List of privileges to assign to the role.
        
        '''
        return self.delegate("AddAuthorizationRole")(name, privIds)
    
    def HasPrivilegeOnEntity(self, entity, sessionId, privId=None):
        '''Check whether a session holds a set of privileges on a managed entity.Check
        whether a session holds a set of privileges on a managed entity.
        
        :param entity: The entity on which the privileges are checked.
        
        :param sessionId: The session ID to check privileges for. A sesssion ID can be obtained from key.
        
        :param privId: The array of privilege IDs to check.
        
        '''
        return self.delegate("HasPrivilegeOnEntity")(entity, sessionId, privId)
    
    def MergePermissions(self, srcRoleId, dstRoleId):
        '''Reassigns all permissions of a role to another role.
        
        :param srcRoleId: The ID of the source role providing the permissions which are changing.
        
        :param dstRoleId: The ID of the destination role to which the permissions are reassigned.
        
        '''
        return self.delegate("MergePermissions")(srcRoleId, dstRoleId)
    
    def RemoveAuthorizationRole(self, roleId, failIfUsed):
        '''Removes a role.
        
        :param roleId: 
        
        :param failIfUsed: If true, prevents the role from being removed if any permissions are using it.
        
        '''
        return self.delegate("RemoveAuthorizationRole")(roleId, failIfUsed)
    
    def RemoveEntityPermission(self, entity, user, isGroup):
        '''Removes a permission rule from an entity.Removes a permission rule from an
        entity.Removes a permission rule from an entity.
        
        :param entity: Entity on which a permission is removed.
        
        :param user: User or group for which the permission is defined.
        
        :param isGroup: True, if user refers to a group name; false, for a user name.
        
        '''
        return self.delegate("RemoveEntityPermission")(entity, user, isGroup)
    
    def ResetEntityPermissions(self, entity, permission=None):
        '''Update the entire set of permissions defined on an entity. Any existing
        permissions on the entity are removed and replaced with the provided set.Update
        the entire set of permissions defined on an entity. Any existing permissions on
        the entity are removed and replaced with the provided set.Update the entire set
        of permissions defined on an entity. Any existing permissions on the entity are
        removed and replaced with the provided set.Update the entire set of permissions
        defined on an entity. Any existing permissions on the entity are removed and
        replaced with the provided set.Update the entire set of permissions defined on
        an entity. Any existing permissions on the entity are removed and replaced with
        the provided set.Update the entire set of permissions defined on an entity. Any
        existing permissions on the entity are removed and replaced with the provided
        set.
        
        :param entity: The entity on which permissions are updated.
        
        :param permission: The list of Permission objects that define the new rules for access to the entity and potentially entities below it. If the list is empty, all permissions on the entity are removed.
        
        '''
        return self.delegate("ResetEntityPermissions")(entity, permission)
    
    def RetrieveAllPermissions(self):
        '''Finds all permissions defined in the system. The result is restricted to the
        managed entities visible to the user making the call.
        
        '''
        return self.delegate("RetrieveAllPermissions")()
    
    def RetrieveEntityPermissions(self, entity, inherited):
        '''Gets permissions defined on or effective on a managed entity. This returns the
        actual permission objects defined in the system for all users and groups
        relative to the managed entity. The inherited flag specifies whether or not to
        include permissions defined by the parents of this entity that propagate to
        this entity.Gets permissions defined on or effective on a managed entity. This
        returns the actual permission objects defined in the system for all users and
        groups relative to the managed entity. The inherited flag specifies whether or
        not to include permissions defined by the parents of this entity that propagate
        to this entity.Gets permissions defined on or effective on a managed entity.
        This returns the actual permission objects defined in the system for all users
        and groups relative to the managed entity. The inherited flag specifies whether
        or not to include permissions defined by the parents of this entity that
        propagate to this entity.
        
        :param entity: 
        
        :param inherited: Whether or not to include propagating permissions defined by parent entities.
        
        '''
        return self.delegate("RetrieveEntityPermissions")(entity, inherited)
    
    def RetrieveRolePermissions(self, roleId):
        '''Finds all the permissions that use a particular role. The result is restricted
        to managed entities that are visible to the user making the call.
        
        :param roleId: 
        
        '''
        return self.delegate("RetrieveRolePermissions")(roleId)
    
    def SetEntityPermissions(self, entity, permission=None):
        '''Defines one or more permission rules on an entity or updates rules if already
        present for the given user or group on the entity.Defines one or more
        permission rules on an entity or updates rules if already present for the given
        user or group on the entity.Defines one or more permission rules on an entity
        or updates rules if already present for the given user or group on the
        entity.Defines one or more permission rules on an entity or updates rules if
        already present for the given user or group on the entity.Defines one or more
        permission rules on an entity or updates rules if already present for the given
        user or group on the entity.
        
        :param entity: The entity on which to set permissions.
        
        :param permission: An array of specifications for permissions on the entity.
        
        '''
        return self.delegate("SetEntityPermissions")(entity, permission)
    
    def UpdateAuthorizationRole(self, roleId, newName, privIds=None):
        '''Updates a role's name or privileges. If the new set of privileges are assigned
        to the role, the system-defined privileges, "System.Anonymous", "System.View",
        and "System.Read" will be assigned to the role too.
        
        :param roleId: The ID of the role that is updated.
        
        :param newName: The new name for the role.
        
        :param privIds: The new set of privileges to assign to the role.
        
        '''
        return self.delegate("UpdateAuthorizationRole")(roleId, newName, privIds)



########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

VirtualDeviceURIBackingOptionDirection = Enum(
    'client',
    'server',

)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

LicenseManagerLicenseKey = Enum(
    'backup',
    'das',
    'drs',
    'drsPower',
    'esxExpress',
    'esxFull',
    'esxHost',
    'esxVmtn',
    'gsxHost',
    'iscsi',
    'nas',
    'san',
    'serverHost',
    'vc',
    'vcExpress',
    'vmotion',
    'vsmp',

)




from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.mo.extensible_managed_object import ExtensibleManagedObject

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class Alarm(ExtensibleManagedObject):
    '''This managed object type defines an alarm that is triggered and an action that
    occurs due to the triggered alarm when certain conditions are met on a specific
    ManagedEntity object.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.Alarm):
        super(Alarm, self).__init__(core, name=name, ref=ref, type=type)

    
    @property
    def info(self):
        '''Information about this alarm.'''
        return self.update('info')

    
    
    def ReconfigureAlarm(self, spec):
        '''Reconfigures the alarm properties. This operation requires access privileges on
        the entity with which the alarm is associated.Reconfigures the alarm
        properties. This operation requires access privileges on the entity with which
        the alarm is associated.
        
        :param spec: The new specification for the alarm.
        
        '''
        return self.delegate("ReconfigureAlarm")(spec)
    
    def RemoveAlarm(self):
        '''Removes the alarm.
        
        '''
        return self.delegate("RemoveAlarm")()



########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

DasVmPriority = Enum(
    'disabled',
    'high',
    'low',
    'medium',

)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

LicenseFeatureInfoSourceRestriction = Enum(
    'file',
    'served',
    'unrestricted',

)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

EntityReferenceEntityType = Enum(
    'cluster',
    'datacenter',
    'datastore',
    'host',
    'nasMount',
    'resourcePool',
    'scsiAdapter',
    'scsiPath',
    'scsiTarget',
    'scsiVolume',
    'storagePod',
    'vm',
    'vmFile',

)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

StateAlarmOperator = Enum(
    'isEqual',
    'isUnequal',

)




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class NetworkFirewallRulesetAllowedip(Base):
    '''
    Commands to list and add/remove allowedip on ruleset
    '''
    moid = 'ha-cli-handler-network-firewall-ruleset-allowedip'
    def add(self, ipaddress, rulesetid):
        '''
        Add allowed ip address/range to the ruleset ruleset.
        :param ipaddress: string, Allowed ip address/range for the ruleset.
        :param rulesetid: string, The label of the ruleset.
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.network.firewall.ruleset.allowedip.Add',
                            ipaddress=ipaddress,
                            rulesetid=rulesetid,
                            )
    def list(self, rulesetid=None):
        '''
        list allowed ip addresses for rulesets.
        :param rulesetid: string, The label of the ruleset.
        :returns: vim.EsxCLI.network.firewall.ruleset.allowedip.list.FirewallRulesetAllowedip[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.network.firewall.ruleset.allowedip.List',
                            rulesetid=rulesetid,
                            )
    def remove(self, ipaddress, rulesetid):
        '''
        Remove allowed ip address/range from the ruleset.
        :param ipaddress: string, Allowed ip address/range for the ruleset.
        :param rulesetid: string, The label of the ruleset.
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.network.firewall.ruleset.allowedip.Remove',
                            ipaddress=ipaddress,
                            rulesetid=rulesetid,
                            )   



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def PhysicalNicCdpInfo(vim, *args, **kwargs):
    '''CDP (Cisco Discovery Protocol) is a link level protocol that allows for
    discovering the CDP-awared network hardware at either end of a DIRECT
    connection. It's only good for direct connection because CDP doesn't get
    forwarded through switches. It's a simple advertisement protocol which beacons
    information about the switch or host along with some port information. The CDP
    information allows ESX Server admins to know which Cisco switch port is
    connected to any given virtual switch uplink (PNIC).'''

    obj = vim.client.factory.create('{urn:vim25}PhysicalNicCdpInfo')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 0:
        raise IndexError('Expected at least 1 arguments got: %d' % len(args))

    required = [  ]
    optional = [ 'address', 'cdpVersion', 'deviceCapability', 'devId', 'fullDuplex',
        'hardwarePlatform', 'ipPrefix', 'ipPrefixLen', 'location', 'mgmtAddr', 'mtu',
        'portId', 'samples', 'softwareVersion', 'systemName', 'systemOID', 'timeout',
        'ttl', 'vlan', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def TooManyGuestLogons(vim, *args, **kwargs):
    '''A TooManyGuestLogons exception is thrown when there are too many concurrent
    login sessions active in the guest. ReleaseCredentialsInGuest can be called on
    ticketed sessions that are no longer needed. This will decrease the number of
    concurrent sessions active in the guest.'''

    obj = vim.client.factory.create('{urn:vim25}TooManyGuestLogons')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 4:
        raise IndexError('Expected at least 5 arguments got: %d' % len(args))

    required = [ 'dynamicProperty', 'dynamicType', 'faultCause', 'faultMessage' ]
    optional = [  ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def OvfCreateImportSpecResult(vim, *args, **kwargs):
    '''The CreateImportSpecResult contains all information regarding the import that
    can be extracted from the OVF descriptor.For example, this includes the list of
    items that the caller must upload in order to complete the import, but not the
    list of URLs to which the files must be uploaded. These paths are not known
    until the VMs have been created, ie. until ResourcePool.importVApp has been
    called.'''

    obj = vim.client.factory.create('{urn:vim25}OvfCreateImportSpecResult')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 0:
        raise IndexError('Expected at least 1 arguments got: %d' % len(args))

    required = [  ]
    optional = [ 'error', 'fileItem', 'importSpec', 'warning', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

GuestFileType = Enum(
    'directory',
    'file',
    'symlink',

)




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def VirtualMachineFlagInfo(vim, *args, **kwargs):
    '''The FlagInfo data object type encapsulates the flag settings for a virtual
    machine. These properties are optional since the same structure is used to
    change the values during an edit or create operation.'''

    obj = vim.client.factory.create('{urn:vim25}VirtualMachineFlagInfo')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 0:
        raise IndexError('Expected at least 1 arguments got: %d' % len(args))

    required = [  ]
    optional = [ 'disableAcceleration', 'diskUuidEnabled', 'enableLogging', 'htSharing',
        'monitorType', 'recordReplayEnabled', 'runWithDebugInfo', 'snapshotDisabled',
        'snapshotLocked', 'snapshotPowerOffBehavior', 'useToe', 'virtualExecUsage',
        'virtualMmuUsage', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class IscsiAdapterAuthChap(Base):
    '''
    Operations that can be performed on iSCSI adapter CHAP authentication
    '''
    moid = 'ha-cli-handler-iscsi-adapter-auth-chap'
    def set(self, adapter, authname=None, default=None, direction=None, level=None, secret=None):
        '''
        Set the iSCSI CHAP authentication for the iSCSI Host Bus Adapter.
        :param adapter: string, The iSCSI adapter name.
        :param authname: string, The iSCSI CHAP authentication name
        :param default: boolean, Resetting iSCSI CHAP authenthication setting to default.
        :param direction: string, The iSCSI CHAP authentication direction ( [uni, mutual])
        :param level: string, The iSCSI CHAP authentication level ( [prohibited, discouraged, preferred, required])
        :param secret: string, The iSCSI CHAP authentication secret
        :returns: void
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.iscsi.adapter.auth.chap.Set',
                            adapter=adapter,
                            authname=authname,
                            default=default,
                            direction=direction,
                            level=level,
                            secret=secret,
                            )
    def get(self, adapter, direction=None):
        '''
        Get the iSCSI CHAP authentication for the iSCSI Host Bus Adapter.
        :param adapter: string, The iSCSI adapter name.
        :param direction: string, The iSCSI CHAP authentication direction ( [uni, mutual])
        :returns: vim.EsxCLI.iscsi.adapter.auth.chap.get.AdapterChapAuth[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.iscsi.adapter.auth.chap.Get',
                            adapter=adapter,
                            direction=direction,
                            )   



########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

DVPortStatusVmDirectPathGen2InactiveReasonNetwork = Enum(
    'portNptDisabledForPort',
    'portNptIncompatibleDvs',
    'portNptNoCompatibleNics',
    'portNptNoVirtualFunctionsAvailable',

)




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class StorageNmpSatpRule(Base):
    '''
    Operations pertaining to claimrules for Storage Array Type Plug-ins.
    '''
    moid = 'ha-cli-handler-storage-nmp-satp-rule'
    def add(self, satp, boot=None, claimoption=None, description=None, device=None, driver=None, force=None, model=None, option=None, psp=None, pspoption=None, transport=None, type=None, vendor=None):
        '''
        Add a rule to the list of claim rules for the given SATP.
        :param boot: boolean, This is a system default rule added at boot time. Do not modify esx.conf or add to host profile.
        :param claimoption: string, Set the claim option string when adding a SATP claim rule.
        :param description: string, Set the claim rule description when adding a SATP claim rule.
        :param device: string, Set the device when adding SATP claim rules. Device rules are mutually exclusive with vendor/model and driver rules.
        :param driver: string, Set the driver string when adding a SATP claim rule. Driver rules are mutually exclusive with vendor/model rules.
        :param force: boolean, Force claim rules to ignore validity checks and install the rule anyway.
        :param model: string, Set the model string when adding SATP a claim rule. Vendor/Model rules are mutually exclusive with driver rules.
        :param option: string, Set the option string when adding a SATP claim rule.
        :param psp: string, Set the default PSP for the SATP claim rule.
        :param pspoption: string, Set the PSP options for the SATP claim rule.
        :param satp: string, The SATP for which a new rule will be added.
        :param transport: string, Set the claim transport type string when adding a SATP claim rule.
        :param type: string, Set the claim type when adding a SATP claim rule.
        :param vendor: string, Set the vendor string when adding SATP claim rules. Vendor/Model rules are mutually exclusive with driver rules.
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.storage.nmp.satp.rule.Add',
                            boot=boot,
                            claimoption=claimoption,
                            description=description,
                            device=device,
                            driver=driver,
                            force=force,
                            model=model,
                            option=option,
                            psp=psp,
                            pspoption=pspoption,
                            satp=satp,
                            transport=transport,
                            type=type,
                            vendor=vendor,
                            )
    def list(self, satp=None):
        '''
        List the claiming rules for Storage Array Type Plugins (SATP)
        :param satp: string, Filter the SATP rules to a specific SATP
        :returns: vim.EsxCLI.storage.nmp.satp.rule.list.StorageArrayTypePluginRule[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.storage.nmp.satp.rule.List',
                            satp=satp,
                            )
    def remove(self, satp, boot=None, claimoption=None, description=None, device=None, driver=None, model=None, option=None, psp=None, pspoption=None, transport=None, type=None, vendor=None):
        '''
        Delete a rule from the list of claim rules for the given SATP.
        :param boot: boolean, This is a system default rule added at boot time. Do not modify esx.conf or add to host profile.
        :param claimoption: string, The claim option string for the SATP claim rule to delete.
        :param description: string, The desription string for the SATP claim rule to delete.
        :param device: string, The device for the SATP claim rule to delete
        :param driver: string, The driver string for the SATP claim rule to delete.
        :param model: string, The model string for the SATP claim rule to delete.
        :param option: string, The option string for the SATP claim rule to delete.
        :param psp: string, The default PSP for the SATP claim rule to delete.
        :param pspoption: string, The PSP options for the SATP claim rule to delete.
        :param satp: string, The SATP for which a rule will be deleted.
        :param transport: string, The transport type for the SATP claim rule to delete.
        :param type: string, Set the claim type when adding a SATP claim rule.
        :param vendor: string, The vendor string for the SATP claim rule to delete
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.storage.nmp.satp.rule.Remove',
                            boot=boot,
                            claimoption=claimoption,
                            description=description,
                            device=device,
                            driver=driver,
                            model=model,
                            option=option,
                            psp=psp,
                            pspoption=pspoption,
                            satp=satp,
                            transport=transport,
                            type=type,
                            vendor=vendor,
                            )   



########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

ReplicationVmFaultReasonForFault = Enum(
    'invalidInstanceId',
    'invalidState',
    'notConfigured',
    'offlineReplicating',
    'poweredOff',
    'poweredOn',
    'suspended',

)




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class IscsiAdapterTargetPortalParam(Base):
    '''
    Operations that can be performed on iSCSI target portal parameters
    '''
    moid = 'ha-cli-handler-iscsi-adapter-target-portal-param'
    def set(self, adapter, address, key, name, default=None, inherit=None, value=None):
        '''
        Set the iSCSI parameter for the iSCSI Target.
        :param adapter: string, The iSCSI adapter name.
        :param address: string, The iSCSI target address: <ip/dns[:port]>
        :param default: boolean, Resetting iSCSI parameter setting to default.
        :param inherit: boolean, Inheriting iSCSI parameter setting from parent.
        :param key: string, The iSCSI parameter key
        :param name: string, The iSCSI target name: <iqn/eui>
        :param value: string, The iSCSI parameter value
        :returns: void
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.iscsi.adapter.target.portal.param.Set',
                            adapter=adapter,
                            address=address,
                            default=default,
                            inherit=inherit,
                            key=key,
                            name=name,
                            value=value,
                            )
    def get(self, adapter, address, name):
        '''
        Get iSCSI parameter on a target
        :param adapter: string, The iSCSI adapter name.
        :param address: string, The iSCSI target address: <ip/dns[:port]>
        :param name: string, The iSCSI target name: <iqn/eui>
        :returns: vim.EsxCLI.iscsi.adapter.target.portal.param.get.iScsiParm[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.iscsi.adapter.target.portal.param.Get',
                            adapter=adapter,
                            address=address,
                            name=name,
                            )   



from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class IscsiPhysicalnetworkportalParam(Base):
    '''
    Operations that can be performed on iSCSI Physical Network Portal (vmnic)'s parameters
    '''
    moid = 'ha-cli-handler-iscsi-physicalnetworkportal-param'
    def set(self, adapter, enabled, option, nic=None):
        '''
        Set network parameter on a Physical Network Portal
        :param adapter: string, The iSCSI adapter name.
        :param enabled: boolean, Enable or disable a Physical Network Portal option
        :param nic: string, The physical network portal name: <vmnic>
        :param option: string, The network parameter option
        :returns: void
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.iscsi.physicalnetworkportal.param.Set',
                            adapter=adapter,
                            enabled=enabled,
                            nic=nic,
                            option=option,
                            )
    def get(self, adapter, nic=None):
        '''
        Get network parameters on a Physical Network Portal (vmnic)
        :param adapter: string, The iSCSI adapter name.
        :param nic: string, The physical network portal name: <vmnic>
        :returns: vim.EsxCLI.iscsi.physicalnetworkportal.param.get.iScsiNetParm[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.iscsi.physicalnetworkportal.param.Get',
                            adapter=adapter,
                            nic=nic,
                            )   



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def HostFibreChannelOverEthernetTargetTransport(vim, *args, **kwargs):
    '''Fibre Channel Over Ethernet transport information about a SCSI target. FCoE
    transport information is that of: the regular FC World Wide Node and Port
    Names; the VNPort MAC address and FCF MAC address which constitute a VN_Port to
    VF_Port Virtual Link; and the VLAN on which an FCoE target resides. More FCoE
    information can be found in the working draft of the T11's Fibre Channel
    Backbone 5 standard (FC-BB-5). The draft can be found at http://www.t11.org.'''

    obj = vim.client.factory.create('{urn:vim25}HostFibreChannelOverEthernetTargetTransport')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 5:
        raise IndexError('Expected at least 6 arguments got: %d' % len(args))

    required = [ 'fcfMac', 'vlanId', 'vnportMac', 'nodeWorldWideName', 'portWorldWideName' ]
    optional = [ 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class StorageCoreDeviceDetached(Base):
    '''
    Operations pertaining to the pluggable storage architectures' detached logical devices on the system.
    '''
    moid = 'ha-cli-handler-storage-core-device-detached'
    def list(self, device=None):
        '''
        Lists all devices that were detached manually by changing their state on the system.
        :param device: string, Filter the output of the command to limit the output to a specific device.
        :returns: vim.EsxCLI.storage.core.device.detached.list.ScsiDetachedDeviceListMap[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.storage.core.device.detached.List',
                            device=device,
                            )
    def remove(self, device):
        '''
        Provide control to allow a user to remove Detached devices from the persistent detached device list.
        :param device: string, Select the detached device to remove from the Detached Device List.
        :returns: void
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.storage.core.device.detached.Remove',
                            device=device,
                            )   



from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class NetworkVswitchStandard(Base):
    '''
    Commands to list and manipulate Legacy Virtual Switches on an ESX host.
    '''
    moid = 'ha-cli-handler-network-vswitch-standard'
    def add(self, vswitchname, ports=None):
        '''
        Add a new virtual switch to the ESXi networking system.
        :param ports: long, The number of ports to to give this newly created virtual switch. Maximum ports per virtual switch is 4096. If no value is given the default value(128) is used. The number of ports is limited by the number of already allocated ports on the host. The system wide port count cannot be greater than 4608.
        :param vswitchname: string, The name of the virtual switch to create.
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.network.vswitch.standard.Add',
                            ports=ports,
                            vswitchname=vswitchname,
                            )
    def set(self, vswitchname, cdpstatus=None, mtu=None):
        '''
        This command sets the MTU size and CDP status of a given virtual switch.
        :param cdpstatus: string, The CDP status of the given virtual switch. It can be 'down', 'listen', 'advertise' or 'both'
        :param mtu: long, The MTU size of the given virtual switch. 
        :param vswitchname: string, The name of virtual switch to apply the configurations.
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.network.vswitch.standard.Set',
                            cdpstatus=cdpstatus,
                            mtu=mtu,
                            vswitchname=vswitchname,
                            )
    def list(self, vswitchname=None):
        '''
        List the virtual switches current on the ESXi host.
        :param vswitchname: string, Limit the output of this command to only virtual switches with the given name.
        :returns: vim.EsxCLI.network.vswitch.standard.list.VirtualSwitch[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.network.vswitch.standard.List',
                            vswitchname=vswitchname,
                            )
    def remove(self, vswitchname):
        '''
        Remove a virtual switch from the ESXi networking system.
        :param vswitchname: string, The name of the virtual switch to remove.
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.network.vswitch.standard.Remove',
                            vswitchname=vswitchname,
                            )   



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def HttpNfcLeaseDeviceUrl(vim, *args, **kwargs):
    '''Provides a mapping from logical device IDs to upload/download URLs.For export,
    a single device id is returned based on the object identifiers for the
    objects.For import, two device ids are returned. One based on the object names
    used in the ImportSpec, and one based on the object identifiers for the created
    objects. This is immutable and would match the id if an ExportLease is latter
    created.'''

    obj = vim.client.factory.create('{urn:vim25}HttpNfcLeaseDeviceUrl')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 4:
        raise IndexError('Expected at least 5 arguments got: %d' % len(args))

    required = [ 'importKey', 'key', 'sslThumbprint', 'url' ]
    optional = [ 'datastoreKey', 'disk', 'fileSize', 'targetId', 'dynamicProperty',
        'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

HostCpuPowerManagementInfoPolicyType = Enum(
    'dynamicPolicy',
    'off',
    'staticPolicy',

)




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class IscsiSoftware(Base):
    '''
    Operations that can be performed on software iSCSI
    '''
    moid = 'ha-cli-handler-iscsi-software'
    def set(self, enabled):
        '''
        Enable or disable software iSCSI.
        :param enabled: boolean, Enable or disable the module.
        :returns: string
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.iscsi.software.Set',
                            enabled=enabled,
                            )
    def get(self):
        '''
        Software iSCSI information.
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.iscsi.software.Get',
                            )   



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def NoLicenseServerConfigured(vim, *args, **kwargs):
    '''The NoLicenseServerConfigured fault is thrown when there is no valid license
    server configured for the system and the system is not in evaluation mode. Any
    operation occurs that requires evaluation license or a valid license will throw
    the NoLicenseServerConfigured. This can happen with the new licensing scheme
    that is a hybrid of flex-based licensing and serial number based licensing.
    There can be cases where VirtualCenter is licensed by a serial number and there
    is no need for a flex license server. These cases are valid as long as no
    operation that requires flex- based license server is invoked, for example,
    adding a pre-4.0 host that requires flex licenses. If however, such an
    operation is invoked, the NoLicenseServerConfigured fault is thrown.'''

    obj = vim.client.factory.create('{urn:vim25}NoLicenseServerConfigured')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 4:
        raise IndexError('Expected at least 5 arguments got: %d' % len(args))

    required = [ 'dynamicProperty', 'dynamicType', 'faultCause', 'faultMessage' ]
    optional = [  ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def EventDescriptionEventDetail(vim, *args, **kwargs):
    '''Each Event object provides an automatic event message string through its
    fullFormattedMessage property. However, you can use the EventDetail object's
    properties to format an event message string that is appropriate when viewed
    from a specific context. The variable information (vm.name, and so on) is
    derived from the Event object's event arguments (VmEventArgument, and so on).'''

    obj = vim.client.factory.create('{urn:vim25}EventDescriptionEventDetail')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 7:
        raise IndexError('Expected at least 8 arguments got: %d' % len(args))

    required = [ 'category', 'formatOnComputeResource', 'formatOnDatacenter', 'formatOnHost',
        'formatOnVm', 'fullFormat', 'key' ]
    optional = [ 'description', 'longDescription', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

NetIpConfigInfoIpAddressOrigin = Enum(
    'dhcp',
    'linklayer',
    'manual',
    'other',
    'random',

)



'''
Created on Jul 29, 2011

@author: eplaster
'''
import re
from munch import Munch

def camel_to_under(string):
    s1 = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', string)
    return re.sub('([a-z0-9])([A-Z])', r'\1_\2', s1).lower()

def munchify(*sequence):
    """[item_a, item_b, ... ] -> munch(item_a=item_a, item_b=item_b, ....)"""
    enums = Munch({item:item for item in sequence})
    return enums




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def VirtualUSB(vim, *args, **kwargs):
    '''The VirtualUSB data object describes the USB device configuration for a virtual
    machine. You can attach a USB device to an ESX host. The device is available to
    only one virtual machine at a time. When you remove the device from the virtual
    machine, it becomes available to other virtual machines located on the host.
    You can add up to 20 USB devices to a virtual machine. Virtual USB support
    requires virtual machine hardware version 7 or later.The VirtualUSB object
    represents either a configuration to be applied to the virtual machine or the
    current device configuration on the virtual machine.To configure a USB device
    for a virtual machine, the virtual machine must have a USB controller. To add a
    controller, include a VirtualUSBController object in the virtual device
    specification for your virtual machine configuration. You can add only one USB
    controller to a virtual machine.To determine the USB options available on the
    host, use the QueryConfigOption method to retrieve the virtual machine
    configuration. The presence of the VirtualUSBOption object in the retrieved
    configuration (VirtualMachineConfigOption.hardwareOptions.virtualDeviceOption)
    indicates that the host supports USB connections.The following operations will
    disconnect a USB device, losing data if data transfer is in progress over the
    USB connection.The following services do not support USB connections.'''

    obj = vim.client.factory.create('{urn:vim25}VirtualUSB')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 2:
        raise IndexError('Expected at least 3 arguments got: %d' % len(args))

    required = [ 'connected', 'key' ]
    optional = [ 'family', 'product', 'speed', 'vendor', 'backing', 'connectable',
        'controllerKey', 'deviceInfo', 'unitNumber', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

HostDasErrorEventHostDasErrorReason = Enum(
    'agentFailed',
    'agentShutdown',
    'communicationInitFailed',
    'configFailed',
    'healthCheckScriptFailed',
    'isolationAddressUnpingable',
    'other',
    'timeout',

)




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class NetworkVswitchStandardPortgroup(Base):
    '''
    Commands to list and manipulate Port Groups on an ESX host.
    '''
    moid = 'ha-cli-handler-network-vswitch-standard-portgroup'
    def add(self, portgroupname, vswitchname):
        '''
        Allows the addition of a standard port group to a virtual switch.
        :param portgroupname: string, The name of the port group to add
        :param vswitchname: string, The virtual switch to add the port group to.
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.network.vswitch.standard.portgroup.Add',
                            portgroupname=portgroupname,
                            vswitchname=vswitchname,
                            )
    def set(self, portgroupname, vlanid=None):
        '''
        Set the vlan id for the given port group
        :param portgroupname: string, The name of the port group to set vlan id for.
        :param vlanid: long, The vlan id for this port group. This value is in the range (0 - 4095)
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.network.vswitch.standard.portgroup.Set',
                            portgroupname=portgroupname,
                            vlanid=vlanid,
                            )
    def list(self):
        '''
        List all of the port groups currently on the system.
        :returns: vim.EsxCLI.network.vswitch.standard.portgroup.list.PortGroup[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.network.vswitch.standard.portgroup.List',
                            )
    def remove(self, portgroupname, vswitchname):
        '''
        Remove a port group from the given virtual switch
        :param portgroupname: string,  
        :param vswitchname: string,  
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.network.vswitch.standard.portgroup.Remove',
                            portgroupname=portgroupname,
                            vswitchname=vswitchname,
                            )   



########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

VirtualMachineStandbyActionType = Enum(
    'checkpoint',
    'powerOnSuspend',

)




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class SystemVisorfsTardisk(Base):
    '''
    Operations pertaining to visorfs tardisks.
    '''
    moid = 'ha-cli-handler-system-visorfs-tardisk'
    def list(self):
        '''
        List the tardisks used by the host.
        :returns: vim.EsxCLI.system.visorfs.tardisk.list.VisorfsTardisk[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.system.visorfs.tardisk.List',
                            )   



from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.mo.history_collector import HistoryCollector

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class EventHistoryCollector(HistoryCollector):
    '''EventHistoryCollector provides a mechanism for retrieving historical data and
    updates when the server appends new events.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.EventHistoryCollector):
        super(EventHistoryCollector, self).__init__(core, name=name, ref=ref, type=type)

    
    @property
    def latestPage(self):
        '''The items in the 'viewable latest page'. As new events that match the
        collector's EventFilterSpec are created, they are added to this page, and the
        oldest events are removed from the collector to keep the size of the page to
        that allowed by HistoryCollector#setLatestPageSize.'''
        return self.update('latestPage')

    
    
    def ReadNextEvents(self, maxCount):
        '''Reads the 'scrollable view' from the current position. The scrollable position
        is moved to the next newer page after the read. No item is returned when the
        end of the collector is reached.
        
        :param maxCount: The maximum number of items in the page.
        
        '''
        return self.delegate("ReadNextEvents")(maxCount)
    
    def ReadPreviousEvents(self, maxCount):
        '''Reads the 'scrollable view' from the current position. The scrollable position
        is moved to the next older page after the read. No item is returned when the
        head of the collector is reached.
        
        :param maxCount: The maximum number of items in the page.
        
        '''
        return self.delegate("ReadPreviousEvents")(maxCount)



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def SessionManagerGenericServiceTicket(vim, *args, **kwargs):
    '''This data object represents a ticket which grants access to some service. The
    ticket may be used only once and is valid only for the
    SessionManagerServiceRequestSpec with which it was acquired. For HTTP service
    requests (when spec is of type HttpServiceRequestSpec) the returned ticket must
    be used by setting id as the value of a special cookie in the HTTP request. For
    CGI requests the name of this cookie is 'vmware_cgi_ticket'. The use of the
    returned ticket for other services is to be defined.'''

    obj = vim.client.factory.create('{urn:vim25}SessionManagerGenericServiceTicket')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 1:
        raise IndexError('Expected at least 2 arguments got: %d' % len(args))

    required = [ 'id' ]
    optional = [ 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def ExtensionOvfConsumerInfo(vim, *args, **kwargs):
    '''This data object contains configuration for extensions that also extend the OVF
    functionality of vCenter server.This feature is experimental in this version of
    the API and might change in future versions.'''

    obj = vim.client.factory.create('{urn:vim25}ExtensionOvfConsumerInfo')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 2:
        raise IndexError('Expected at least 3 arguments got: %d' % len(args))

    required = [ 'callbackUrl', 'sectionType' ]
    optional = [ 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class SoftwareSourcesProfile(Base):
    '''
    Query depot contents image profiles
    '''
    moid = 'ha-cli-handler-software-sources-profile'
    def list(self, depot, proxy=None):
        '''
        List all the image profiles in a depot.
        :param depot: string[], Specifies full remote URLs of the depot index.xml or server file path pointing to an offline bundle .zip file.
        :param proxy: string, Specifies a proxy server to use for HTTP, FTP, and HTTPS connections. The format is proxy-url:port.
        :returns: vim.EsxCLI.software.sources.profile.list.ImageProfileSummary[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.software.sources.profile.List',
                            depot=depot,
                            proxy=proxy,
                            )
    def get(self, depot, profile, proxy=None):
        '''
        Display details about an image profile from the depot.
        :param depot: string[], Specifies full remote URLs of the depot index.xml or server file path pointing to an offline bundle .zip file.
        :param profile: string, Specifies the name of the image profile to display.
        :param proxy: string, Specifies a proxy server to use for HTTP, FTP, and HTTPS connections. The format is proxy-url:port.
        :returns: vim.EsxCLI.software.sources.profile.get.ImageProfile
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.software.sources.profile.Get',
                            depot=depot,
                            profile=profile,
                            proxy=proxy,
                            )   



from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.base.base_entity import BaseEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class HostLocalAccountManager(BaseEntity):
    '''This managed object type provides an interface through which local accounts on
    a host are managed. Note that this managed object applies only to applications
    that use a local account database on the host to provide authentication (ESX
    Server, for example). POSIX and win32 hosts may impose different restrictions
    on the password, ID, and description formats. POSIX host implementation may
    restrict the user or group name to be lower case letters and less than 16
    characters in total. It may also disallow characters such as ";", "\n", and so
    on. In short, all the platform dependent rules and restrictions regarding
    naming of users/groups and password apply here. An InvalidArgument fault is
    thrown if any of these rules are not obeyed.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.HostLocalAccountManager):
        super(HostLocalAccountManager, self).__init__(core, name=name, ref=ref, type=type)

    

    
    
    def AssignUserToGroup(self, user, group):
        '''Assigns a user to a group.
        
        :param user: User ID of the account whose group membership is being assigned.
        
        :param group: Destination group account to which the user is being assigned.
        
        '''
        return self.delegate("AssignUserToGroup")(user, group)
    
    def CreateGroup(self, group):
        '''Creates a local group account using the parameters defined in the
        HostLocalAccountManagerAccountSpecification data object type. For POSIX hosts,
        passing the HostLocalAccountManagerPosixAccountSpecification data object type
        allows you to control the group ID format of the group account being created.
        
        :param group: Specification of group being created.
        
        '''
        return self.delegate("CreateGroup")(group)
    
    def CreateUser(self, user):
        '''Creates a local user account using the parameters defined in the
        HostLocalAccountManagerAccountSpecification data object type. For POSIX hosts,
        passing HostLocalAccountManagerPosixAccountSpecification data object type
        allows you to control the format of the user ID of the user account being
        created.
        
        :param user: Specification of user being created.
        
        '''
        return self.delegate("CreateUser")(user)
    
    def RemoveGroup(self, groupName):
        '''Removes a local group account.
        
        :param groupName: Group ID of the group account being removed.
        
        '''
        return self.delegate("RemoveGroup")(groupName)
    
    def RemoveUser(self, userName):
        '''Removes a local user account.
        
        :param userName: User ID of the user account being removed.
        
        '''
        return self.delegate("RemoveUser")(userName)
    
    def UnassignUserFromGroup(self, user, group):
        '''Unassigns a user from a group.
        
        :param user: User being unassigned from group.
        
        :param group: Group from which the user is being removed.
        
        '''
        return self.delegate("UnassignUserFromGroup")(user, group)
    
    def UpdateUser(self, user):
        '''Updates a local user account using the parameters defined in the
        HostLocalAccountManagerAccountSpecification data object type.
        
        :param user: Specification of user being updated.
        
        '''
        return self.delegate("UpdateUser")(user)



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def HostMultipathInfo(vim, *args, **kwargs):
    '''The HostMultipathInfo data object describes the multipathing policy
    configuration to determine the storage failover policies for a SCSI logical
    unit. The multipathing policy configuration operates on SCSI logical units and
    the paths to the logical units.Multipath policy configuration is only possible
    on storage devices provided by the native multipathing plug-store plugin.
    Storage devices using the native multipathing storage plugin will have an entry
    in this data object. Storage devices provided by a different storage plugin
    will not appear in the inventory represented by this data object.Legacy note:
    In hosts where HostMultipathStateInfo is not defined or does not exist on the
    HostStorageDeviceInfo object, only native multipathing exists. That means for
    these hosts, the MultipathInfo object contains the complete set of LUNs and
    paths on the LUNs available on the host.'''

    obj = vim.client.factory.create('{urn:vim25}HostMultipathInfo')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 0:
        raise IndexError('Expected at least 1 arguments got: %d' % len(args))

    required = [  ]
    optional = [ 'lun', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def VirtualMachineVMCIDevice(vim, *args, **kwargs):
    '''The VirtualMachineVMCIDevice data object represents a virtual communication
    device that supports the VMCI (Virtual Machine Communication Interface). Each
    virtual machine has a VMCI device that handles interprocess socket-based
    communication. VMCI device information is available in the virtual machine
    hardware device list (VirtualMachine.config.hardware.device[]).An application
    running on a virtual machine uses the VMCI Sockets API for communication with
    other virtual machines on the same host, or for communication with the host.
    For information about using the vSphere VMCI Sockets API, see the .'''

    obj = vim.client.factory.create('{urn:vim25}VirtualMachineVMCIDevice')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 1:
        raise IndexError('Expected at least 2 arguments got: %d' % len(args))

    required = [ 'key' ]
    optional = [ 'allowUnrestrictedCommunication', 'id', 'backing', 'connectable',
        'controllerKey', 'deviceInfo', 'unitNumber', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

NetIpConfigInfoIpAddressStatus = Enum(
    'deprecated',
    'duplicate',
    'inaccessible',
    'invalid',
    'preferred',
    'tentative',
    'unknown',

)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

CannotMoveFaultToleranceVmMoveType = Enum(
    'cluster',
    'resourcePool',

)




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def HostNetworkConfig(vim, *args, **kwargs):
    '''This data object type describes networking host configuration data objects.
    These objects contain only the configuration information for networking. The
    runtime information is available from the NetworkInfo data object type.See
    HostNetworkInfo'''

    obj = vim.client.factory.create('{urn:vim25}HostNetworkConfig')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 0:
        raise IndexError('Expected at least 1 arguments got: %d' % len(args))

    required = [  ]
    optional = [ 'consoleIpRouteConfig', 'consoleVnic', 'dhcp', 'dnsConfig', 'ipRouteConfig',
        'ipV6Enabled', 'nat', 'pnic', 'portgroup', 'proxySwitch', 'routeTableConfig',
        'vnic', 'vswitch', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

HostDiskPartitionInfoType = Enum(
    'extended',
    'linuxNative',
    'linuxSwap',
    'none',
    'ntfs',
    'vmfs',
    'vmkDiagnostic',

)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

HostReplayUnsupportedReason = Enum(
    'cpuidLimitSet',
    'hvDisabled',
    'incompatibleCpu',
    'incompatibleProduct',
    'oldBIOS',
    'unknown',

)




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class IscsiAdapterParam(Base):
    '''
    Operations that can be performed on iSCSI adapter parameters
    '''
    moid = 'ha-cli-handler-iscsi-adapter-param'
    def set(self, adapter, key, default=None, value=None):
        '''
        Set the iSCSI parameter for the iSCSI Host Bus Adapter.
        :param adapter: string, The iSCSI adapter name.
        :param default: boolean, Resetting iSCSI parameter setting to default.
        :param key: string, The iSCSI initiator parameter key.
        :param value: string, The iSCSI initiator parameter value.
        :returns: void
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.iscsi.adapter.param.Set',
                            adapter=adapter,
                            default=default,
                            key=key,
                            value=value,
                            )
    def get(self, adapter):
        '''
        Get the iSCSI parameters for the iSCSI Host Bus Adapter.
        :param adapter: string, The iSCSI adapter name.
        :returns: vim.EsxCLI.iscsi.adapter.param.get.iScsiParm[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.iscsi.adapter.param.Get',
                            adapter=adapter,
                            )   



from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.base.base_entity import BaseEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class SmsServiceInstance(BaseEntity):
    '''Service interface for the Storage Monitoring Service.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.SmsServiceInstance):
        super(SmsServiceInstance, self).__init__(core, name=name, ref=ref, type=type)

    

    
    
    def ConfigureSyncInterval(self, interval):
        '''Sets the frequency with which the service cache is automatically synced with
        provider information. The interval is specified in seconds with a minimum value
        of 300 (5 minutes). NOTE: In certain concurrent invocation scenarios, it is
        possible that the sync timer will not be scheduled with the latest configured
        value until one further sync cycle has completed.
        
        :param interval: sync interval in seconds (>= 300).
        
        '''
        return self.delegate("ConfigureSyncInterval")(interval)
    
    def QueryAboutInfo(self):
        '''Retrieves information about the service.
        
        '''
        return self.delegate("QueryAboutInfo")()
    
    def QueryList(self, contextEntity, queryEntityType, querySpec):
        '''Returns a result set (an instance of the QueryResult data object) containing <a
        href="result-entities.html">properties</a> of all entity instances of the
        specified entity type. If the optional contextEntity parameter is supplied, the
        result set is limited to those entities related to the specified entity. See
        this <a href="result-contextEntity_relationships.html">matrix for lists of
        possible relationships.</a> You can apply additional constraints to the results
        by using the QuerySpec input parameter.
        
        :param contextEntity: Limits result set from the query to storage attributes associated with the specified entity.
        
        :param queryEntityType: Type of entities to query.
        
        :param querySpec: Additional constraints (range, filtering, sorting, and so on) to be applied on the result set.
        
        '''
        return self.delegate("QueryList")(contextEntity, queryEntityType, querySpec)
    
    def QueryStorageManager(self):
        '''Retrieves Storage Manager managed object.
        
        '''
        return self.delegate("QueryStorageManager")()
    
    def QueryTopology(self, entity):
        '''<b>Deprecated.</b> <i>As of SMS API 2.0, use sms.ServiceInstance.queryList API
        to obtain storage topology information.</i> Returns the entities that are
        related to the given entity as a list of nodes and edges, which can be used to
        construct a topological map of the relationships. Entities supported by this
        operation: datacenter, cluster, host, datastore and vm.
        
        :param entity: entity for which topology map is required.
        
        '''
        return self.delegate("QueryTopology")(entity)
    
    def Sync(self):
        '''Synchronizes the service cache from provider information (like the
        VirtualCenter database provider).
        
        '''
        return self.delegate("Sync")()
    
    def UpdateVcDbConnectionInfo(self, dbConnectionSpec):
        '''Updates the connection information for the VirtualCenter database. The
        credentials and connection url can be explicitly specified using a
        DbConnectionSpec argument. Alternately, a null argument may be passed to force
        the service to refresh the connection information from the local registry.
        NOTE: The operation re-initializes the service after configuring the database
        information. This is akin to the service being restarted.
        
        :param dbConnectionSpec: database connection information.
        
        '''
        return self.delegate("UpdateVcDbConnectionInfo")(dbConnectionSpec)



########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

VasaProviderProfile = Enum(
    'blockDevice',
    'capability',
    'fileSystem',

)




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def NamePasswordAuthentication(vim, *args, **kwargs):
    '''NamePasswordAuthentication contains the information necessary to authenticate
    within a guest using a name and password. This is the typical method for
    authentication within a guest and the one currently used by VIX. This method of
    authentication is stateless.To use NamePasswordAuthentication, populate
    username and password with the appropriate login information. You should not
    use AcquireCredentialsInGuest or ReleaseCredentialsInGuest for
    NamePasswordAuthentication.Once populated, you can use
    NamePasswordAuthentication in any guest operations function call.'''

    obj = vim.client.factory.create('{urn:vim25}NamePasswordAuthentication')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 3:
        raise IndexError('Expected at least 4 arguments got: %d' % len(args))

    required = [ 'password', 'username', 'interactiveSession' ]
    optional = [ 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def ScsiLun(vim, *args, **kwargs):
    '''The ScsiLun data object describes a SCSI logical unit. A SCSI logical unit is a
    host device that an ESX Server or virtual machine can use for I/O operations.An
    ESX Server creates SCSI logical unit objects to represent devices in the host
    configuration. (See the definition of ScsiLunType for a list of the supported
    device types.) The vSphere API uses one of two object types to represent a SCSI
    logical unit, depending on the device type.When the Server creates a
    HostScsiDisk or ScsiLun object, it specifies a valid device name and type:'''

    obj = vim.client.factory.create('{urn:vim25}ScsiLun')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 5:
        raise IndexError('Expected at least 6 arguments got: %d' % len(args))

    required = [ 'lunType', 'operationalState', 'uuid', 'deviceName', 'deviceType' ]
    optional = [ 'alternateName', 'canonicalName', 'capabilities', 'descriptor', 'displayName',
        'durableName', 'key', 'model', 'queueDepth', 'revision', 'scsiLevel',
        'serialNumber', 'standardInquiry', 'vendor', 'vStorageSupport',
        'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def CannotModifyConfigCpuRequirements(vim, *args, **kwargs):
    '''A virtual machine's total CPU feature requirements are determined by overlaying
    the requirements specified in its configuration (if any) on top of the
    requirements specified in the descriptor for its guest OS. It is therefore
    possible for a host change to implicitly change a virtual machine's CPU feature
    requirements. The guest OS descriptor may have different requirements on the
    new host. Or, if the virtual machine currently specifies requirements in its
    configuration, those requirements will be lost if the new host does not support
    this.This fault indicates that the virtual machine's CPU feature requirements
    would change because of a migration, and also that the destination host does
    not support storing CPU feature requirements in the virtual machine's
    configuration. (If the destination host does support such an action,
    WillModifyConfigCpuRequirements is used instead of this fault.)For a powered-
    off virtual machine, this is a warning. The migration may proceed, but the
    virtual machine will be operating under different CPU feature requirements if
    it is powered on after the migration.For a powered-on or suspended virtual
    machine, this is an error.'''

    obj = vim.client.factory.create('{urn:vim25}CannotModifyConfigCpuRequirements')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 4:
        raise IndexError('Expected at least 5 arguments got: %d' % len(args))

    required = [ 'dynamicProperty', 'dynamicType', 'faultCause', 'faultMessage' ]
    optional = [  ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.mo.extensible_managed_object import ExtensibleManagedObject

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class HostCpuSchedulerSystem(ExtensibleManagedObject):
    '''This managed object provides an interface through which you can gather and
    configure the host CPU scheduler policies that affect the performance of
    running virtual machines.: This managed object is useful only on platforms
    where resource management controls are available to optimize the running of
    virtual machines.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.HostCpuSchedulerSystem):
        super(HostCpuSchedulerSystem, self).__init__(core, name=name, ref=ref, type=type)

    
    @property
    def hyperthreadInfo(self):
        '''The hyperthread configuration for the CpuSchedulerSystem. The existence of this
        data object type indicates if the CPU scheduler is capable of scheduling
        hyperthreads as resources.'''
        return self.update('hyperthreadInfo')

    
    
    def DisableHyperThreading(self):
        '''Don't treat hyperthreads as schedulable resources the next time the CPU
        scheduler starts. If successful, this operation will change the configured
        setting.
        
        '''
        return self.delegate("DisableHyperThreading")()
    
    def EnableHyperThreading(self):
        '''Treat hyperthreads as schedulable resources the next time the CPU scheduler
        starts. If successful, this operation will set the config property to "true".
        
        '''
        return self.delegate("EnableHyperThreading")()



########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

DisallowedChangeByServiceDisallowedChange = Enum(
    'hotExtendDisk',

)




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def AutoStartPowerInfo(vim, *args, **kwargs):
    '''This object type describes the power-on / power-off behavior for a given
    virtual machine. Virtual machines can be configured to wait for a period of
    time before starting or to wait to receive a successful heartbeat from a
    virtual machine before starting the next virtual machine in the sequence.If
    startAction and stopAction for a virtual machine are both set to none, that
    virtual machine is removed from the AutoStart sequence. Virtual machines can be
    configured both to wait for a period of time before starting and to wait for a
    heartbeat. In such a case, the waiting virtual machine only waits until either
    of these conditions are met. In other words, a virtual machine starts in either
    of the following cases:This provides a better experience since as soon as one
    virtual machine begins sending heartbeats, indicating it has successfully
    started up, the next machine will begin starting up. This happens even if the
    startDelay has not yet elapsed. Similarly, if one virtual machine fails to
    begin sending heartbeats, perhaps because it could not start up, other machines
    are not blocked from starting up since the startDelay eventually elapses.'''

    obj = vim.client.factory.create('{urn:vim25}AutoStartPowerInfo')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 7:
        raise IndexError('Expected at least 8 arguments got: %d' % len(args))

    required = [ 'key', 'startAction', 'startDelay', 'startOrder', 'stopAction', 'stopDelay',
        'waitForHeartbeat' ]
    optional = [ 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def VMwareDVSConfigInfo(vim, *args, **kwargs):
    '''This class defines the VMware specific configuration for
    DistributedVirtualSwitch.'''

    obj = vim.client.factory.create('{urn:vim25}VMwareDVSConfigInfo')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 13:
        raise IndexError('Expected at least 14 arguments got: %d' % len(args))

    required = [ 'maxMtu', 'configVersion', 'contact', 'createTime', 'defaultPortConfig',
        'maxPorts', 'name', 'networkResourceManagementEnabled', 'numPorts',
        'numStandalonePorts', 'productInfo', 'uplinkPortPolicy', 'uuid' ]
    optional = [ 'ipfixConfig', 'linkDiscoveryProtocolConfig', 'pvlanConfig', 'vspanSession',
        'description', 'extensionKey', 'host', 'policy', 'switchIpAddress',
        'targetInfo', 'uplinkPortgroup', 'vendorSpecificConfig', 'dynamicProperty',
        'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

IscsiPortInfoPathStatus = Enum(
    'active',
    'lastActive',
    'notUsed',
    'standBy',

)




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class StorageVmfsSnapshot(Base):
    '''
    Manage VMFS snapshots.
    '''
    moid = 'ha-cli-handler-storage-vmfs-snapshot'
    def resignature(self, volumelabel=None, volumeuuid=None):
        '''
        Resignature a snapshot/replica of a VMFS volume.
        :param volumelabel: string, The VMFS volume label of the snapshot to resignature.
        :param volumeuuid: string, The VMFS volume uuid of the snapshot to resignature.
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.storage.vmfs.snapshot.Resignature',
                            volumelabel=volumelabel,
                            volumeuuid=volumeuuid,
                            )
    def mount(self, nopersist=None, volumelabel=None, volumeuuid=None):
        '''
        Mount a snapshot/replica of a VMFS volume.
        :param nopersist: boolean, Mount the volume non-peristently; the volume will not be automounted after a restart.
        :param volumelabel: string, The VMFS volume label of the snapshot to mount.
        :param volumeuuid: string, The VMFS volume uuid of the snapshot to mount.
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.storage.vmfs.snapshot.Mount',
                            nopersist=nopersist,
                            volumelabel=volumelabel,
                            volumeuuid=volumeuuid,
                            )
    def list(self, volumelabel=None, volumeuuid=None):
        '''
        List unresolved snapshots/replicas of VMFS volume.
        :param volumelabel: string, The VMFS volume label of the snapshot to list.
        :param volumeuuid: string, The VMFS volume uuid of the snapshot to list.
        :returns: vim.EsxCLI.storage.vmfs.snapshot.list.VMFSSnapshot[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.storage.vmfs.snapshot.List',
                            volumelabel=volumelabel,
                            volumeuuid=volumeuuid,
                            )   



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def MismatchedNetworkPolicies(vim, *args, **kwargs):
    '''The virtual machine network uses different offload or security policies on the
    destination host than on the source host. This is an error if the virtual
    machine is currently connected to the network, and a warning otherwise.'''

    obj = vim.client.factory.create('{urn:vim25}MismatchedNetworkPolicies')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 7:
        raise IndexError('Expected at least 8 arguments got: %d' % len(args))

    required = [ 'backing', 'connected', 'device', 'dynamicProperty', 'dynamicType',
        'faultCause', 'faultMessage' ]
    optional = [  ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def AllVirtualMachinesLicensedEvent(vim, *args, **kwargs):
    '''This event records that the previously unlicensed virtual machines on the
    specified host are now licensed. After this event is entered into the event
    log, we expect to see that the (@link
    vim.event.Event.UnlicensedVirtualMachinesEvent UnlicensedVirtualMachinesEvent)
    (@link vim.ManagedEntity.configIssue configIssue) is removed from the host.'''

    obj = vim.client.factory.create('{urn:vim25}AllVirtualMachinesLicensedEvent')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 4:
        raise IndexError('Expected at least 5 arguments got: %d' % len(args))

    required = [ 'chainId', 'createdTime', 'key', 'userName' ]
    optional = [ 'changeTag', 'computeResource', 'datacenter', 'ds', 'dvs',
        'fullFormattedMessage', 'host', 'net', 'vm', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class NetworkIpInterfaceIpv4(Base):
    '''
    Commands to get and set IPv4 settings for vmknic.
    '''
    moid = 'ha-cli-handler-network-ip-interface-ipv4'
    def set(self, interfacename, type, ipv4=None, netmask=None, peerdns=None):
        '''
        Configure IPv4 setting for a given VMkernel network interface.
        :param interfacename: string, The name of the VMkernel network interface to set IPv4 settings for. This name must be an interface listed in the interface list command.
        :param ipv4: string, The static IPv4 address for this interface.
        :param netmask: string, The static IPv4 netmask for this interface.
        :param peerdns: boolean, A boolean value to indicate if the system should use the DNS settings published via DHCP for this interface.
        :param type: string, IPv4 Address type : 
    dhcp: Use DHCP to aquire IPv4 setting for this interface.
    none: Remove IPv4 settings form this interface.
    static: Set Static IPv4 information for this interface. Requires --ipv4 and --netmask options.

        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.network.ip.interface.ipv4.Set',
                            interfacename=interfacename,
                            ipv4=ipv4,
                            netmask=netmask,
                            peerdns=peerdns,
                            type=type,
                            )
    def get(self, interfacename=None):
        '''
        Get IPv4 settings for VMkernel network interfaces.
        :param interfacename: string, The name of the VMkernel network interface to limit the output of this command to.
        :returns: vim.EsxCLI.network.ip.interface.ipv4.get.IPv4Interface[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.network.ip.interface.ipv4.Get',
                            interfacename=interfacename,
                            )   



########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

ProfileNumericComparator = Enum(
    'equal',
    'greaterThan',
    'greaterThanEqual',
    'lessThan',
    'lessThanEqual',
    'notEqual',

)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

PhysicalNicResourcePoolSchedulerDisallowedReason = Enum(
    'hardwareUnsupported',
    'userOptOut',

)




from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.base.base_entity import BaseEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class ScheduledTaskManager(BaseEntity):
    '''Object manager for scheduled tasks.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.ScheduledTaskManager):
        super(ScheduledTaskManager, self).__init__(core, name=name, ref=ref, type=type)

    
    @property
    def description(self):
        '''Static descriptive strings used in scheduled tasks.'''
        return self.update('description')
    @property
    def scheduledTask(self):
        '''All available scheduled tasks.'''
        return self.update('scheduledTask')

    
    
    def CreateObjectScheduledTask(self, obj, spec):
        '''Creates a scheduled task.
        
        :param obj: The managed object for which the scheduled task triggers an action. You can schedule tasks on any managed object.
        
        :param spec: The specification for the new scheduled task.
        
        '''
        return self.delegate("CreateObjectScheduledTask")(obj, spec)
    
    def CreateScheduledTask(self, entity, spec):
        '''Creates a scheduled task.
        
        :param entity: The managed entity (or entities) for which the scheduled task triggers an action. You can schedule tasks on any managed entity. If the scheduled task is associated with a leaf node in the inventory tree, it applies only to a single entity (virtual machine or host). If the task is associated with a folder, a datacenter, a compute resource, or a resource pool, it applies to the virtual machine or host descendants of the entity.
        
        :param spec: The specification for the new scheduled task.
        
        '''
        return self.delegate("CreateScheduledTask")(entity, spec)
    
    def RetrieveEntityScheduledTask(self, entity=None):
        '''Available scheduled tasks defined on the entity.
        
        :param entity: The entity. If null, all scheduled tasks are returned for visible entities.
        
        '''
        return self.delegate("RetrieveEntityScheduledTask")(entity)
    
    def RetrieveObjectScheduledTask(self, obj=None):
        '''Available scheduled tasks defined on the object.
        
        :param obj: The object. If not specified, all scheduled tasks are returned for visible entities and visible ManagedObjects.
        
        '''
        return self.delegate("RetrieveObjectScheduledTask")(obj)



from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.base.base_entity import BaseEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class HostAuthenticationManager(BaseEntity):
    '''The HostAuthenticationManager managed object provides access to Active
    Directory configuration information for an ESX host. It also provides access to
    methods for adding a host to or removing a host from an Active Directory
    domain.The vSphere API supports Microsoft Active Directory management of
    authentication for ESX hosts. To integrate an ESX host into an Active Directory
    environment, you use an Active Directory account that has the authority to add
    a computer to a domain. The ESX Server locates the Active Directory domain
    controller. When you add a host to a domain, you only need to specify the
    domain and the account user name and password.There are two approaches that you
    can use to add an ESX host to or remove a host from an Active Directory
    domain.* JoinDomain_Task and LeaveCurrentDomain_Task methods - Your vSphere
    client application can call these methods directly to add the host to or remove
    the host from a domain. * Host configuration - Use the HostActiveDirectory data
    object to specify Active Directory configuration, either adding the host to or
    removing the host from a domain. To apply the Active Directory configuration,
    use the HostProfileManager method (ApplyHostConfig_Task) to apply the
    HostConfigSpec. When the ESX Server processes the configuration, it will invoke
    the join or leave method.To take advantage of ESX host membership in an Active
    Directory domain, grant permissions on the ESX host to Active Directory users
    and groups who should have direct access to management of the ESX host. Use the
    UserDirectory.RetrieveUserGroups method to obtain information about Active
    Directory users and groups. After retrieving the Active Directory data, you can
    use the AuthorizationManager.SetEntityPermissions method to set the principal
    property to the appropriate user or group.By default, the ESX host assigns the
    Administrator role to the "ESX Admins" group. If the group does not exist when
    the host joins the domain, the host will not assign the role. In this case, you
    must create the "ESX Admins" group in the Active Directory. The host will
    periodically check the domain controller for the group and will assign the role
    when the group exists.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.HostAuthenticationManager):
        super(HostAuthenticationManager, self).__init__(core, name=name, ref=ref, type=type)

    
    @property
    def info(self):
        '''Information about Active Directory membership.'''
        return self.update('info')
    @property
    def supportedStore(self):
        '''An array that can contain managed object references to local and Active
        Directory authentication managed objects.'''
        return self.update('supportedStore')

    



########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

VirtualMachineUsbInfoSpeed = Enum(
    'full',
    'high',
    'low',
    'superSpeed',
    'unknownSpeed',

)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

VirtualMachineMemoryAllocationPolicy = Enum(
    'swapMost',
    'swapNone',
    'swapSome',

)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

PerfSummaryType = Enum(
    'average',
    'latest',
    'maximum',
    'minimum',
    'none',
    'summation',

)




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def NasStorageProfile(vim, *args, **kwargs):
    '''The NasStorageProfile data object represents one NAS datastore configuration.
    Use the policy list for access to configuration data for the NAS storage
    profile. Use the property list for access to subprofile configuration data, if
    any.'''

    obj = vim.client.factory.create('{urn:vim25}NasStorageProfile')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 2:
        raise IndexError('Expected at least 3 arguments got: %d' % len(args))

    required = [ 'key', 'enabled' ]
    optional = [ 'policy', 'profileTypeName', 'profileVersion', 'property', 'dynamicProperty',
        'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.base.base_entity import BaseEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class HostSnmpSystem(BaseEntity):
    '''Provision the SNMP Version 1,2c agent. This object is accessed through the
    HostConfigManager object.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.HostSnmpSystem):
        super(HostSnmpSystem, self).__init__(core, name=name, ref=ref, type=type)

    
    @property
    def configuration(self):
        '''VI API 2.5'''
        return self.update('configuration')
    @property
    def limits(self):
        '''VI API 2.5'''
        return self.update('limits')

    
    
    def ReconfigureSnmpAgent(self, spec):
        '''
        
        :param spec: 
        
        '''
        return self.delegate("ReconfigureSnmpAgent")(spec)
    
    def SendTestNotification(self):
        '''
        
        '''
        return self.delegate("SendTestNotification")()



from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class NetworkFence(Base):
    '''
    Commands to list fence information
    '''
    moid = 'ha-cli-handler-network-fence'
    def list(self):
        '''
        Get all fence switch info on the system.
        :returns: vim.EsxCLI.network.fence.list.FenceSwitch[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.network.fence.List',
                            )   



from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.base.base_entity import BaseEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class FileManager(BaseEntity):
    '''This managed object type provides a way to manage and manipulate files and
    folders on datastores. The source and the destination names are in the form of
    a URL or a datastore path.A URL has the formwhere* is or . * specifies the
    hostname or IP address of the VirtualCenter or ESX server and optionally the
    port. * is the inventory path to the Datacenter containing the Datastore. * is
    the name of the Datastore. * is a slash-delimited path from the root of the
    datastore.A datastore path has the formwhere* is the datastore name. * is a
    slash-delimited path from the root of the datastore.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.FileManager):
        super(FileManager, self).__init__(core, name=name, ref=ref, type=type)

    

    
    
    def ChangeOwner(self, name, owner, datacenter=None):
        '''Change the owner for a file.Change the owner for a file.
        
        :param name: 
        
        :param datacenter: 
        
        :param owner: 
        
        '''
        return self.delegate("ChangeOwner")(name, datacenter, owner)
    
    def CopyDatastoreFile_Task(self, sourceName, destinationName, sourceDatacenter=None, destinationDatacenter=None, force=None):
        '''Copies the source file or folder to the destination.Copies the source file or
        folder to the destination.Copies the source file or folder to the
        destination.Copies the source file or folder to the destination.Copies the
        source file or folder to the destination.Copies the source file or folder to
        the destination.Copies the source file or folder to the destination.Copies the
        source file or folder to the destination.Copies the source file or folder to
        the destination.
        
        :param sourceName: The name of the source, either a URL or a datastore path referring to the file or folder to be copied.
        
        :param sourceDatacenter: Ifis a datastore path, the datacenter for that datastore path. Not needed when invoked directly on ESX. If not specified on a call to VirtualCenter,must be a URL.
        
        :param destinationName: The name of the destination, either a URL or a datastore path referring to the destination file or folder.
        
        :param destinationDatacenter: Ifis a datastore path, the datacenter for that datastore path. Not needed when invoked directly on ESX. If not specified on a call to VirtualCenter, it is assumed that the destination path belongs to the source datacenter.
        
        :param force: If true, overwrite any identically named file at the destination. If not specified, it is assumed to be false.
        
        '''
        return self.delegate("CopyDatastoreFile_Task")(sourceName, sourceDatacenter, destinationName, destinationDatacenter, force)
    
    def DeleteDatastoreFile_Task(self, name, datacenter=None):
        '''Deletes the specified file or folder from the datastore. If a file of a virtual
        machine is deleted, it may corrupt that virtual machine. Folder deletes are
        always recursive. The datacenter parameter may be omitted if a URL is used to
        name the file or folder.Deletes the specified file or folder from the
        datastore. If a file of a virtual machine is deleted, it may corrupt that
        virtual machine. Folder deletes are always recursive. The datacenter parameter
        may be omitted if a URL is used to name the file or folder.Deletes the
        specified file or folder from the datastore. If a file of a virtual machine is
        deleted, it may corrupt that virtual machine. Folder deletes are always
        recursive. The datacenter parameter may be omitted if a URL is used to name the
        file or folder.Deletes the specified file or folder from the datastore. If a
        file of a virtual machine is deleted, it may corrupt that virtual machine.
        Folder deletes are always recursive. The datacenter parameter may be omitted if
        a URL is used to name the file or folder.
        
        :param name: The name of the file or folder, either a URL or a datastore path referring to the file or folder to be deleted.
        
        :param datacenter: Ifis a datastore path, the datacenter for that datastore path. Not needed when invoked directly on ESX. If not specified on a call to VirtualCenter,must be a URL.
        
        '''
        return self.delegate("DeleteDatastoreFile_Task")(name, datacenter)
    
    def MakeDirectory(self, name, datacenter=None, createParentDirectories=None):
        '''Create a folder using the specified name. If the parent or intermediate level
        folders do not exist, and the parameter createParentDirectories is false, a
        FileNotFound fault is thrown. If the intermediate level folders do not exist,
        and the parameter createParentDirectories is true, all the non-existent folders
        are created.Create a folder using the specified name. If the parent or
        intermediate level folders do not exist, and the parameter
        createParentDirectories is false, a FileNotFound fault is thrown. If the
        intermediate level folders do not exist, and the parameter
        createParentDirectories is true, all the non-existent folders are created.
        
        :param name: The name of the folder, either a URL or a datastore path referring to the folder to be created.
        
        :param datacenter: Ifis a datastore path, the datacenter for that datastore path. Not needed when invoked directly on ESX. If not specified on a call to VirtualCenter,must be a URL.
        
        :param createParentDirectories: If true, any non-existent intermediate level folders will be created. If not specified, it is assumed to be false.
        
        '''
        return self.delegate("MakeDirectory")(name, datacenter, createParentDirectories)
    
    def MoveDatastoreFile_Task(self, sourceName, destinationName, sourceDatacenter=None, destinationDatacenter=None, force=None):
        '''Moves the source file or folder to the destination.Moves the source file or
        folder to the destination.Moves the source file or folder to the
        destination.Moves the source file or folder to the destination.Moves the source
        file or folder to the destination.Moves the source file or folder to the
        destination.Moves the source file or folder to the destination.Moves the source
        file or folder to the destination.Moves the source file or folder to the
        destination.
        
        :param sourceName: The name of the source, either a URL or a datastore path referring to the file or folder to be moved.
        
        :param sourceDatacenter: Ifis a datastore path, the datacenter for that datastore path. Not needed when invoked directly on ESX. If not specified on a call to VirtualCenter,must be a URL.
        
        :param destinationName: The name of the destination, either a URL or a datastore path referring to the destination file or folder.
        
        :param destinationDatacenter: Ifis a datastore path, the datacenter for that datastore path. Not needed when invoked directly on ESX. If not specified on a call to VirtualCenter, it is assumed that the destination path belongs to the source datacenter.
        
        :param force: If true, overwrite any identically named file at the destination. If not specified, it is assumed to be false.
        
        '''
        return self.delegate("MoveDatastoreFile_Task")(sourceName, sourceDatacenter, destinationName, destinationDatacenter, force)


from sys import exc_info
from logging import getLogger
from infi.pyutils.decorators import wraps
from infi.pyutils.contexts import contextmanager
from infi.pyutils.lazy import cached_method, clear_cache
from .errors import CreateTaskException
from ..extension import ExtensionFacade

# The only valuable reference for how to use custom tasks is:
# http://www.doublecloud.org/2010/09/creating-your-own-task-and-event-in-vsphere/

"""
This facade allows developers to easily create Tasks in vSphere.

To create tasks, use the TaskManager:

>>> manager = vim.getTaskManager("myapp")
>>> with manager.task("Doing this"):
...     pass
"""

logger = getLogger(__name__)

class TaskManager(object):
    def __init__(self, client, extension_key=__name__):
        super(TaskManager, self).__init__()
        self._client = client
        self._managed_object = self._client.service_content.taskManager
        self._extension = ExtensionFacade(self._client, extension_key)

    def task(self, managed_object, name, parent=None):
        if not self._extension.is_registered():
            self._extension.register()
        if not self._extension.is_task_registered(name):
            self._extension.register_task(name)
        task_id = self._extension.get_task_id(name)
        task_managed_object = self._create_task(managed_object, task_id, parent)
        task = Task(self._client, task_managed_object)
        return task

    def _create_task(self, managed_object, type_id, parent=None):
        kwargs = dict(obj=managed_object, taskTypeId=type_id, cancelable=False, initiatedBy=None)
        kwargs['parentTaskKey'] = getattr(parent, 'get_id', lambda: None)()
        logger.info("Creating task {!r}".format(kwargs))
        try:
            task_managed_object = self._managed_object.CreateTask(**kwargs).task
        except:
            logger.exception("Failed to create task")
            raise CreateTaskException, None, exc_info()[-1]
        return task_managed_object

class Task(object):
    def __init__(self, client, managed_object):
        super(Task, self).__init__()
        self._client = client
        self._managed_object = managed_object

    def update_progress(self, percentage):
        self._managed_object.UpdateProgress(percentage)

    def step(self, steps):
        for _ in self.iter(steps):
            pass

    def iter(self, steps):
        percentage = 0
        increment = 100 / len(steps)
        with self:
            for step in steps:
                step()
                percentage += increment
                self.update_progress(percentage)
                yield percentage
            self.update_progress(100)

    def wraps(self, func):
        @wraps(func)
        def callable(*args, **kwargs):
            with self:
                return func(*args, **kwargs)
        return callable

    @contextmanager
    def silent_context(self):
        try:
            with self:
                yield
        except Exception:
            pass

    def __enter__(self):
        self.set_state('running')
        return self

    def __exit__(self, *exc_info):
        exception_has_been_raised = exc_info != (None, None, None)
        if exception_has_been_raised:
            self.set_state('error')
            self.set_description(exc_info[1].message)
            logger.error("Exception has been raised inside the task context", exc_info=exc_info)
            raise exc_info[0], exc_info[1], exc_info[2]
        self.set_state('success')
        self.set_description("")

    def get_name(self):
        data_object = self._get_info()
        return data_object.name or ''

    def set_state(self, state):
        from pyvisdk.do.localized_method_fault import LocalizedMethodFault
        from pyvisdk.do.vim_fault import VimFault
        localized_method_fault = None
        if state == 'error':
            fault = VimFault(self._client, [], [], None, [])
            localized_method_fault = LocalizedMethodFault(self._client, fault=fault, localizedMessage='error')
        self._managed_object.SetTaskState(state, None, localized_method_fault)

    def set_description(self, message):
        from pyvisdk.do.localizable_message import LocalizableMessage
        msg = LocalizableMessage(self._client, key=self._managed_object.info.key, message=message)
        self._managed_object.SetTaskDescription(msg)

    def get_id(self):
        return self._get_info().key

    def _get_info(self):
        return self._managed_object.info

    def __repr__(self):
        return "<Task id={!r}>".format(self.get_id())





import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def VirtualPCNet32Option(vim, *args, **kwargs):
    '''The VirtualPCNet32Option data object type defines the options for the
    VirtualPCNet32 data object type. Except for the boolean supportsMorphing
    option, the options are inherited from the VirtualEthernetCardOption data
    object type.'''

    obj = vim.client.factory.create('{urn:vim25}VirtualPCNet32Option')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 9:
        raise IndexError('Expected at least 10 arguments got: %d' % len(args))

    required = [ 'supportsMorphing', 'macType', 'supportedOUI', 'vmDirectPathGen2Supported',
        'wakeOnLanEnabled', 'deprecated', 'hotRemoveSupported', 'plugAndPlay', 'type' ]
    optional = [ 'autoAssignController', 'backingOption', 'connectOption', 'controllerType',
        'defaultBackingOptionIndex', 'licensingLimit', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class SystemWelcomemsg(Base):
    '''
    Commands to get and set the welcome banner for DCUI.
    '''
    moid = 'ha-cli-handler-system-welcomemsg'
    def set(self, message):
        '''
        Set the Welcome Message for DCUI.
        :param message: string, Welcome Message String.
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.system.welcomemsg.Set',
                            message=message,
                            )
    def get(self):
        '''
        Get the Welcome Message for DCUI.
        :returns: string
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.system.welcomemsg.Get',
                            )   



########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

HostSnmpAgentCapability = Enum(
    'COMPLETE',
    'CONFIGURATION',
    'DIAGNOSTICS',

)




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def HostResignatureRescanResult(vim, *args, **kwargs):
    '''When a user resignatures an UnresolvedVmfsVolume through DatastoreSystem API,
    we resignature and auto-mount on the other hosts which share the same
    underlying storage luns. As part of the operation, we rescan the specified list
    of hosts. This data object describes the return value of resignature APIs in
    DatastoreSystem.'''

    obj = vim.client.factory.create('{urn:vim25}HostResignatureRescanResult')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 1:
        raise IndexError('Expected at least 2 arguments got: %d' % len(args))

    required = [ 'result' ]
    optional = [ 'rescan', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.base.base_entity import BaseEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class VirtualDiskManager(BaseEntity):
    '''This managed object type provides a way to manage and manipulate virtual disks
    on datastores. The source and the destination names are in the form of a URL or
    a datastore path.A URL has the formwhere* is or . * specifies the hostname or
    IP address of the VirtualCenter or ESX server and optionally the port. * is the
    inventory path to the Datacenter containing the Datastore. * is the name of the
    Datastore. * is a slash-delimited path from the root of the datastore.A
    datastore path has the formwhere* is the datastore name. * is a slash-delimited
    path from the root of the datastore.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.VirtualDiskManager):
        super(VirtualDiskManager, self).__init__(core, name=name, ref=ref, type=type)

    

    
    
    def CopyVirtualDisk_Task(self, sourceName, destName, sourceDatacenter=None, destDatacenter=None, destSpec=None, force=None):
        '''Copy a virtual disk, performing conversions as specified in the spec.Copy a
        virtual disk, performing conversions as specified in the spec.Copy a virtual
        disk, performing conversions as specified in the spec.Copy a virtual disk,
        performing conversions as specified in the spec.
        
        :param sourceName: The name of the source, either a datastore path or a URL referring to the virtual disk to be copied.
        
        :param sourceDatacenter: Ifis a datastore path, the datacenter for that datastore path. Not needed when invoked directly on ESX. If not specified on a call to VirtualCenter,must be a URL.
        
        :param destName: The name of the destination, either a datastore path or a URL referring to the virtual disk to be created.
        
        :param destDatacenter: Ifis a datastore path, the datacenter for that datastore path. Not needed when invoked directly on ESX. If not specified on a call to VirtualCenter, it is assumed that the destination path belongs to the source datacenter.
        
        :param destSpec: The specification of the virtual disk to be created. If not specified, a preallocated format and busLogic adapter type is assumed.
        
        :param force: The force flag is currently ignored. The FileAlreadyExists fault is thrown if the destination file already exists.
        
        '''
        return self.delegate("CopyVirtualDisk_Task")(sourceName, sourceDatacenter, destName, destDatacenter, destSpec, force)
    
    def CreateVirtualDisk_Task(self, name, spec, datacenter=None):
        '''Create a virtual disk.Create a virtual disk.Create a virtual disk.
        
        :param name: The name of the disk, either a datastore path or a URL referring to the virtual disk to be created.
        
        :param datacenter: Ifis a datastore path, the datacenter for that datastore path. Not needed when invoked directly on ESX. If not specified on a call to VirtualCenter,must be a URL.
        
        :param spec: The specification of the virtual disk to be created.
        
        '''
        return self.delegate("CreateVirtualDisk_Task")(name, datacenter, spec)
    
    def DefragmentVirtualDisk_Task(self, name, datacenter=None):
        '''Defragment a sparse virtual disk. This is defragmentation of the virtual disk
        file(s) in the host operating system, not defragmentation of the guest
        operating system filesystem inside the virtual disk.Defragment a sparse virtual
        disk. This is defragmentation of the virtual disk file(s) in the host operating
        system, not defragmentation of the guest operating system filesystem inside the
        virtual disk.Defragment a sparse virtual disk. This is defragmentation of the
        virtual disk file(s) in the host operating system, not defragmentation of the
        guest operating system filesystem inside the virtual disk.
        
        :param name: The name of the disk, either a datastore path or a URL referring to the virtual disk that should be defragmented.
        
        :param datacenter: Ifis a datastore path, the datacenter for that datastore path. Not needed when invoked directly on ESX. If not specified on a call to VirtualCenter,must be a URL.
        
        '''
        return self.delegate("DefragmentVirtualDisk_Task")(name, datacenter)
    
    def DeleteVirtualDisk_Task(self, name, datacenter=None):
        '''Delete a virtual disk. All files relating to the disk will be deleted.Delete a
        virtual disk. All files relating to the disk will be deleted.Delete a virtual
        disk. All files relating to the disk will be deleted.
        
        :param name: The name of the disk, either a datastore path or a URL referring to the virtual disk to be deleted.
        
        :param datacenter: Ifis a datastore path, the datacenter for that datastore path. Not needed when invoked directly on ESX. If not specified on a call to VirtualCenter,must be a URL.
        
        '''
        return self.delegate("DeleteVirtualDisk_Task")(name, datacenter)
    
    def EagerZeroVirtualDisk_Task(self, name, datacenter=None):
        '''Explicitly zero out unaccessed parts zeroedthick disk. Effectively a no-op if
        the disk is already eagerZeroedThick. Unlike zeroFillVirtualDisk, which wipes
        the entire disk, this operation only affects previously unaccessed parts of the
        disk.Explicitly zero out unaccessed parts zeroedthick disk. Effectively a no-op
        if the disk is already eagerZeroedThick. Unlike zeroFillVirtualDisk, which
        wipes the entire disk, this operation only affects previously unaccessed parts
        of the disk.Explicitly zero out unaccessed parts zeroedthick disk. Effectively
        a no-op if the disk is already eagerZeroedThick. Unlike zeroFillVirtualDisk,
        which wipes the entire disk, this operation only affects previously unaccessed
        parts of the disk.
        
        :param name: The name of the disk, either a datastore path or a URL referring to the virtual disk that should be inflated.
        
        :param datacenter: Ifis a datastore path, the datacenter for that datastore path. Not needed when invoked directly on ESX. If not specified on a call to VirtualCenter,must be a URL.
        
        '''
        return self.delegate("EagerZeroVirtualDisk_Task")(name, datacenter)
    
    def ExtendVirtualDisk_Task(self, name, newCapacityKb, datacenter=None, eagerZero=None):
        '''Expand the capacity of a virtual disk to the new capacity. If the eagerZero
        flag is not specified, - the extended disk region of a zerothick disk will be
        zeroedthick - the extended disk region of a eagerzerothick disk will be
        eagerzeroedthick - a thin-provisioned disk will always be extended as a thin-
        provisioned disk. If the eagerZero flag TRUE, the extended region of the disk
        will always be eagerly zeroed. If the eagerZero flag FALSE, the extended region
        of a zeroedthick or eagerzeroedthick the disk will not be eagerly zeroed. This
        condition has no effect on a thin source disk.Expand the capacity of a virtual
        disk to the new capacity. If the eagerZero flag is not specified, - the
        extended disk region of a zerothick disk will be zeroedthick - the extended
        disk region of a eagerzerothick disk will be eagerzeroedthick - a thin-
        provisioned disk will always be extended as a thin-provisioned disk. If the
        eagerZero flag TRUE, the extended region of the disk will always be eagerly
        zeroed. If the eagerZero flag FALSE, the extended region of a zeroedthick or
        eagerzeroedthick the disk will not be eagerly zeroed. This condition has no
        effect on a thin source disk.Expand the capacity of a virtual disk to the new
        capacity. If the eagerZero flag is not specified, - the extended disk region of
        a zerothick disk will be zeroedthick - the extended disk region of a
        eagerzerothick disk will be eagerzeroedthick - a thin-provisioned disk will
        always be extended as a thin-provisioned disk. If the eagerZero flag TRUE, the
        extended region of the disk will always be eagerly zeroed. If the eagerZero
        flag FALSE, the extended region of a zeroedthick or eagerzeroedthick the disk
        will not be eagerly zeroed. This condition has no effect on a thin source disk.
        
        :param name: The name of the disk, either a datastore path or a URL referring to the virtual disk whose capacity should be expanded.
        
        :param datacenter: Ifis a datastore path, the datacenter for that datastore path. Not needed when invoked directly on ESX. If not specified on a call to VirtualCenter,must be a URL.
        
        :param newCapacityKb: The new capacty of the virtual disk in Kb.
        
        :param eagerZero: If true, the extended part of the disk will be explicitly filled with zeroes.vSphere API 4.0
        
        '''
        return self.delegate("ExtendVirtualDisk_Task")(name, datacenter, newCapacityKb, eagerZero)
    
    def InflateVirtualDisk_Task(self, name, datacenter=None):
        '''Inflate a sparse or thin-provisioned virtual disk up to the full size.
        Additional space allocated to the disk as a result of this operation will be
        filled with zeroes.Inflate a sparse or thin-provisioned virtual disk up to the
        full size. Additional space allocated to the disk as a result of this operation
        will be filled with zeroes.Inflate a sparse or thin-provisioned virtual disk up
        to the full size. Additional space allocated to the disk as a result of this
        operation will be filled with zeroes.
        
        :param name: The name of the disk, either a datastore path or a URL referring to the virtual disk that should be inflated.
        
        :param datacenter: Ifis a datastore path, the datacenter for that datastore path. Not needed when invoked directly on ESX. If not specified on a call to VirtualCenter,must be a URL.
        
        '''
        return self.delegate("InflateVirtualDisk_Task")(name, datacenter)
    
    def MoveVirtualDisk_Task(self, sourceName, destName, sourceDatacenter=None, destDatacenter=None, force=None):
        '''Move a virtual disk and all related files from the source location specified by
        <code>sourceName</code> and <code>sourceDatacenter</code> to the destination
        location specified by <code>destName</code> and <code>destDatacenter</code>.
        
        :param sourceName: The name of the source, either a datastore path or a URL referring to the virtual disk to be moved.
        
        :param sourceDatacenter: Ifis a datastore path, the datacenter for that datastore path. Not needed when invoked directly on ESX. If not specified on a call to VirtualCenter,must be a URL.
        
        :param destName: The name of the destination, either a datastore path or a URL referring to the destination virtual disk.
        
        :param destDatacenter: Ifis a datastore path, the datacenter for that datastore path. Not needed when invoked directly on ESX. If not specified on a call to VirtualCenter, it is assumed that the destination path belongs to the source datacenter.
        
        :param force: If true, overwrite any indentically named disk at the destination. If not specified, it is assumed to be false
        
        '''
        return self.delegate("MoveVirtualDisk_Task")(sourceName, sourceDatacenter, destName, destDatacenter, force)
    
    def QueryVirtualDiskFragmentation(self, name, datacenter=None):
        '''Return the percentage of fragmentation of the sparse virtual disk. This is the
        fragmentation of virtual disk file(s) in the host operating system, not the
        fragmentation of the guest operating systemS filesystem inside the virtual
        disk.Return the percentage of fragmentation of the sparse virtual disk. This is
        the fragmentation of virtual disk file(s) in the host operating system, not the
        fragmentation of the guest operating systemS filesystem inside the virtual
        disk.Return the percentage of fragmentation of the sparse virtual disk. This is
        the fragmentation of virtual disk file(s) in the host operating system, not the
        fragmentation of the guest operating systemS filesystem inside the virtual
        disk.
        
        :param name: The name of the disk, either a datastore path or a URL referring to the virtual disk for which to return the percentage of fragmentation.
        
        :param datacenter: Ifis a datastore path, the datacenter for that datastore path. Not needed when invoked directly on ESX. If not specified on a call to VirtualCenter,must be a URL.
        
        '''
        return self.delegate("QueryVirtualDiskFragmentation")(name, datacenter)
    
    def QueryVirtualDiskGeometry(self, name, datacenter=None):
        '''Get the disk geometry information for the virtual disk.Get the disk geometry
        information for the virtual disk.Get the disk geometry information for the
        virtual disk.
        
        :param name: The name of the disk, either a datastore path or a URL referring to the virtual disk from which to get geometry information.
        
        :param datacenter: Ifis a datastore path, the datacenter for that datastore path. Not needed when invoked directly on ESX. If not specified on a call to VirtualCenter,must be a URL.
        
        '''
        return self.delegate("QueryVirtualDiskGeometry")(name, datacenter)
    
    def QueryVirtualDiskUuid(self, name, datacenter=None):
        '''Get the virtual disk SCSI inquiry page 0x83 data.Get the virtual disk SCSI
        inquiry page 0x83 data.Get the virtual disk SCSI inquiry page 0x83 data.
        
        :param name: The name of the disk, either a datastore path or a URL referring to the virtual disk from which to get SCSI inquiry page 0x83 data.
        
        :param datacenter: Ifis a datastore path, the datacenter for that datastore path. Not needed when invoked directly on ESX. If not specified on a call to VirtualCenter,must be a URL.
        
        '''
        return self.delegate("QueryVirtualDiskUuid")(name, datacenter)
    
    def SetVirtualDiskUuid(self, name, uuid, datacenter=None):
        '''Set the virtual disk SCSI inquiry page 0x83 data.Set the virtual disk SCSI
        inquiry page 0x83 data.Set the virtual disk SCSI inquiry page 0x83 data.
        
        :param name: The name of the disk, either a datastore path or a URL referring to the virtual disk whose SCSI inquiry page 0x83 data should be set.
        
        :param datacenter: Ifis a datastore path, the datacenter for that datastore path. Not needed when invoked directly on ESX. If not specified on a call to VirtualCenter,must be a URL.
        
        :param uuid: The hex representation of the unique ID for this virtual disk.
        
        '''
        return self.delegate("SetVirtualDiskUuid")(name, datacenter, uuid)
    
    def ShrinkVirtualDisk_Task(self, name, datacenter=None, copy=None):
        '''Shrink a sparse virtual disk.Shrink a sparse virtual disk.Shrink a sparse
        virtual disk.
        
        :param name: The name of the disk, either a datastore path or a URL referring to the virtual disk that should be shrink.
        
        :param datacenter: Ifis a datastore path, the datacenter for that datastore path. Not needed when invoked directly on ESX. If not specified on a call to VirtualCenter,must be a URL.
        
        :param copy: If true or omitted, performs shrink in copy-shrink mode, otherwise shrink in in-place mode.
        
        '''
        return self.delegate("ShrinkVirtualDisk_Task")(name, datacenter, copy)
    
    def ZeroFillVirtualDisk_Task(self, name, datacenter=None):
        '''Overwrite all blocks of the virtual disk with zeros. All data will be
        lost.Overwrite all blocks of the virtual disk with zeros. All data will be
        lost.Overwrite all blocks of the virtual disk with zeros. All data will be
        lost.
        
        :param name: The name of the disk, either a datastore path or a URL referring to the virtual disk whose blocks should be overwritten with zeroes.
        
        :param datacenter: Ifis a datastore path, the datacenter for that datastore path. Not needed when invoked directly on ESX. If not specified on a call to VirtualCenter,must be a URL.
        
        '''
        return self.delegate("ZeroFillVirtualDisk_Task")(name, datacenter)



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def VirtualMachineConfigSummary(vim, *args, **kwargs):
    '''A subset of virtual machine configuration.'''

    obj = vim.client.factory.create('{urn:vim25}VirtualMachineConfigSummary')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 3:
        raise IndexError('Expected at least 4 arguments got: %d' % len(args))

    required = [ 'name', 'template', 'vmPathName' ]
    optional = [ 'annotation', 'cpuReservation', 'ftInfo', 'guestFullName', 'guestId',
        'installBootRequired', 'instanceUuid', 'managedBy', 'memoryReservation',
        'memorySizeMB', 'numCpu', 'numEthernetCards', 'numVirtualDisks', 'product',
        'uuid', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class NetworkIp(Base):
    '''
    Operations that can be performed on vmknics
    '''
    moid = 'ha-cli-handler-network-ip'
    def set(self, ipv6enabled=None):
        '''
        Update global IP settings
        :param ipv6enabled: boolean, Enable or disable IPv6 (Reboot Required)
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.network.ip.Set',
                            ipv6enabled=ipv6enabled,
                            )
    def get(self):
        '''
        Get global IP settings
        :returns: vim.EsxCLI.network.ip.get.IPInfo
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.network.ip.Get',
                            )   



from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class SystemUuid(Base):
    '''
    Get the system UUID
    '''
    moid = 'ha-cli-handler-system-uuid'
    def get(self):
        '''
        Get the system UUID.
        :returns: string
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.system.uuid.Get',
                            )   


# http://stackoverflow.com/questions/1875052/using-paired-certificates-with-urllib2

# We need to login to vCenter server as an extension and this means that
# vCenter needs to be able to see our certificate.
# If we just connect to the vCenter server HTTPS
# reverse proxy at https://<vcenter>/sdk, the certificate is not forwarded to
# the vCenter server endpoint. We will get an HTTPS connection to the reverse
# proxy, but the connection from the reverse proxy to vCenter will be HTTP.
#
# To handle this we need to tunnel all our traffic through the proxy server
# when talking to vCenter.

import urllib2
import suds.client

from logging import getLogger
logger = getLogger(__name__)


try:
    from gevent import sleep
except ImportError:
    from time import sleep


def retry_on_exception(func, seconds_between_retries=1, max_retries=None, exception_predicate=None):
    """ Retries the call if an exception occurrs.
    seconds_between_retries indicates the number of seconds to sleep after the operation fails, before the next retry.
    max_retries specifies the number of times to retry the execution, not including the first call.
    i.e. if max_retries is 1, then the command will be called once, and retried once more. If the command still fails
    after max_retries, the last exception will be raised. If max_retries is None, the command will be retried
    indefinitely.
    exception_predicate is a function that is called with the raised exception, to test the exception if only
    specific exceptions are valid for retry. If the function returns True, the command will be retried, otherwise the
    exception will be raised without a retry. If exception_predicate is None, the command will be retried for all
    exceptions. """
    from infi.pyutils.decorators import wraps

    @wraps(func)
    def retry(*args, **kwargs):
        from time import sleep
        retries = 0
        while True:
            try:
                return func(*args, **kwargs)
            except Exception, error:
                if exception_predicate is not None and not exception_predicate(error):
                    raise
                retries += 1
                if max_retries is not None and retries > max_retries:
                    raise
                logger.warn("function {!r} failed with {!r}, retrying".format(func, error))
                sleep(seconds_between_retries)
    return retry


def retry_on_URLError(func):
    from urllib2 import URLError
    # INFINILAB-82: sometimes FC commands fail on the switch, retry works
    def predicate(error):
        return isinstance(error, URLError)
    # we retry twice, with 2 seconds in between. The predicate filters only the relevant exceptions
    return retry_on_exception(func, 1, 30, predicate)


class HTTPSClientAuthHandler(urllib2.HTTPSHandler):
    def __init__(self, key, cert):
        urllib2.HTTPSHandler.__init__(self)
        self.key = key
        self.cert = cert

    def https_open(self, req):
        # Rather than pass in a reference to a connection class, we pass in
        # a reference to a function which, for all intents and purposes,
        # will behave as a constructor
        return self.do_open(self.getConnection, req)

    def getConnection(self, host, timeout=300):
        from httplib import HTTPSConnection
        return HTTPSConnection(host, key_file=self.key, cert_file=self.cert)


class HttpAuthenticated(suds.client.HttpAuthenticated):
    def __init__(self, certfile=None, keyfile=None, proxy=None, **kwargs):
        suds.client.HttpAuthenticated.__init__(self, **kwargs)
        self._certfile = certfile
        self._keyfile = keyfile
        self._proxy = proxy

    def u2handlers(self):
        handlers = suds.client.HttpAuthenticated.u2handlers(self)
        if self._certfile and self._keyfile:
            handlers.append(urllib2.ProxyHandler({'https':self._proxy}))
            handlers.append(HTTPSClientAuthHandler(self._keyfile,
                                                   self._certfile))
        return handlers




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def VirtualDisk(vim, *args, **kwargs):
    '''This data object type contains information about a disk in a virtual
    machine.The virtual disk backing object types describe the different virtual
    disk backings available. The disk format version in each case describes the
    version of the format that is used.Supported virtual disk backings:'''

    obj = vim.client.factory.create('{urn:vim25}VirtualDisk')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 2:
        raise IndexError('Expected at least 3 arguments got: %d' % len(args))

    required = [ 'capacityInKB', 'key' ]
    optional = [ 'shares', 'storageIOAllocation', 'backing', 'connectable', 'controllerKey',
        'deviceInfo', 'unitNumber', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class NetworkFirewallRulesetRule(Base):
    '''
    Commands to list rules in the ruleset
    '''
    moid = 'ha-cli-handler-network-firewall-ruleset-rule'
    def list(self, rulesetid=None):
        '''
        List the rules of each ruleset in firewall.
        :param rulesetid: string, List rules for specfic ruleset
        :returns: vim.EsxCLI.network.firewall.ruleset.rule.list.FirewallPortRule[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.network.firewall.ruleset.rule.List',
                            rulesetid=rulesetid,
                            )   



from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class SystemProcessStatsLoad(Base):
    '''
    System load averages
    '''
    moid = 'ha-cli-handler-system-process-stats-load'
    def get(self):
        '''
        System load average over the last 1, 5 and 15 minutes.
        :returns: vim.EsxCLI.system.process.stats.load.get.ProcessStatsLoadGet
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.system.process.stats.load.Get',
                            )   



########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

NetIpStackInfoPreference = Enum(
    'high',
    'low',
    'medium',
    'reserved',

)




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class IscsiPhysicalnetworkportal(Base):
    '''
    Operations that can be performed on iSCSI Physical Network Portal (vmnic)
    '''
    moid = 'ha-cli-handler-iscsi-physicalnetworkportal'
    def list(self, adapter=None):
        '''
        List Physical Network Portal for iSCSI Adapter
        :param adapter: string, The iSCSI adapter name.
        :returns: vim.EsxCLI.iscsi.physicalnetworkportal.list.PhysicalNetworkPortal[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.iscsi.physicalnetworkportal.List',
                            adapter=adapter,
                            )   



from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class NetworkFirewall(Base):
    '''
    A set of commands for firewall related operations
    '''
    moid = 'ha-cli-handler-network-firewall'
    def load(self):
        '''
        Load firewall module and rulesets configuration.
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.network.firewall.Load',
                            )
    def unload(self):
        '''
        Allow unload firewall module.
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.network.firewall.Unload',
                            )
    def set(self, defaultaction=None, enabled=None):
        '''
        Set firewall enabled status and default action.
        :param defaultaction: boolean, Set to true to set defaultaction PASS, set to false to DROP.
        :param enabled: boolean, Set to true to enable the firewall, set to false to disable the firewall.
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.network.firewall.Set',
                            defaultaction=defaultaction,
                            enabled=enabled,
                            )
    def refresh(self):
        '''
        Load ruleset configuration for firewall.
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.network.firewall.Refresh',
                            )
    def get(self):
        '''
        Get the firewall status.
        :returns: vim.EsxCLI.network.firewall.get.Firewall
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.network.firewall.Get',
                            )   



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def HostFileSystemVolumeInfo(vim, *args, **kwargs):
    '''The HostFileSystemVolumeInfo data object describes the file system volume
    information for the host.A file system volume refers to a storage abstraction
    that allows files to be created and organized. A host can have multiple file
    system volumes. File system volumes are typically mounted into a file namespace
    that allows all files in mounted file systems to be addressable from the host.A
    file system volume is backed by disk storage. It could span one or more disks
    but need not use an entire disk.A file system volume by definition must be
    mounted on the file system in order to exist.'''

    obj = vim.client.factory.create('{urn:vim25}HostFileSystemVolumeInfo')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 0:
        raise IndexError('Expected at least 1 arguments got: %d' % len(args))

    required = [  ]
    optional = [ 'mountInfo', 'volumeTypeList', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

FileSystemInterface = Enum(
    'nfs',
    'otherFileSystem',

)




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def VirtualHardwareOption(vim, *args, **kwargs):
    '''The VirtualHardwareOption data object contains the options available for all
    virtual devices.'''

    obj = vim.client.factory.create('{urn:vim25}VirtualHardwareOption')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 14:
        raise IndexError('Expected at least 15 arguments got: %d' % len(args))

    required = [ 'deviceListReadonly', 'hwVersion', 'memoryMB', 'numCoresPerSocket', 'numCPU',
        'numCpuReadonly', 'numIDEControllers', 'numPCIControllers',
        'numPS2Controllers', 'numSIOControllers', 'numUSBControllers',
        'numUSBXHCIControllers', 'resourceConfigOption', 'virtualDeviceOption' ]
    optional = [ 'licensingLimit', 'numSupportedWwnNodes', 'numSupportedWwnPorts',
        'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

VAppCloneSpecProvisioningType = Enum(
    'sameAsSource',
    'thick',
    'thin',

)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

ClusterPowerOnVmOption = Enum(
    'OverrideAutomationLevel',
    'ReserveResources',

)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

VirtualMachinePowerOffBehavior = Enum(
    'powerOff',
    'prompt',
    'revert',

)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

VirtualMachineHtSharing = Enum(
    'any',
    'internal',
    'none',

)




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def VirtualMachineConfigSpec(vim, *args, **kwargs):
    '''This data object type encapsulates configuration settings when creating or
    reconfiguring a virtual machine. To support incremental changes, these
    properties are all optional. If an optional property is unset, or any nested
    optional property is unset, the property will not be changed unless 'unset' is
    a valid value for the property. To determine whether 'unset' is a valid value
    for a particular property, refer to the documentation for that property.'''

    obj = vim.client.factory.create('{urn:vim25}VirtualMachineConfigSpec')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 0:
        raise IndexError('Expected at least 1 arguments got: %d' % len(args))

    required = [  ]
    optional = [ 'alternateGuestName', 'annotation', 'bootOptions', 'changeTrackingEnabled',
        'changeVersion', 'consolePreferences', 'cpuAffinity', 'cpuAllocation',
        'cpuFeatureMask', 'cpuHotAddEnabled', 'cpuHotRemoveEnabled', 'deviceChange',
        'extraConfig', 'files', 'firmware', 'flags', 'ftInfo', 'guestAutoLockEnabled',
        'guestId', 'instanceUuid', 'locationId', 'managedBy', 'maxMksConnections',
        'memoryAffinity', 'memoryAllocation', 'memoryHotAddEnabled', 'memoryMB',
        'memoryReservationLockedToMax', 'name', 'networkShaper', 'npivDesiredNodeWwns',
        'npivDesiredPortWwns', 'npivNodeWorldWideName', 'npivOnNonRdmDisks',
        'npivPortWorldWideName', 'npivTemporaryDisabled', 'npivWorldWideNameOp',
        'npivWorldWideNameType', 'numCoresPerSocket', 'numCPUs', 'powerOpInfo',
        'swapPlacement', 'tools', 'uuid', 'vAppConfig', 'vAppConfigRemoved',
        'vAssertsEnabled', 'version', 'virtualICH7MPresent', 'virtualSMCPresent',
        'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def IncompatibleDefaultDevice(vim, *args, **kwargs):
    '''A default device (see defaultDevice for a definition) which the virtual machine
    is using is incompatible with the corresponding default device which will be
    created on the target host.This is an issue with powered-on or suspended
    migration under some circumstances. The problem is that in cases where the
    virtual machine must be recreated, it will have the default device created with
    default settings that are appropriate for the target host. If those are not
    compatible with the settings for that device that the virtual machine is
    currently using, then resuming the virtual machine on the target host might
    fail.This might happen if the device in question were reconfigured or the
    default is different between the source and the destination host. An example of
    a default device and associated setting which might cause this is
    videoRamSizeInKB. This is an error.'''

    obj = vim.client.factory.create('{urn:vim25}IncompatibleDefaultDevice')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 5:
        raise IndexError('Expected at least 6 arguments got: %d' % len(args))

    required = [ 'device', 'dynamicProperty', 'dynamicType', 'faultCause', 'faultMessage' ]
    optional = [  ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

VirtualDeviceConfigSpecFileOperation = Enum(
    'create',
    'destroy',
    'replace',

)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

ComplianceResultStatus = Enum(
    'compliant',
    'nonCompliant',
    'unknown',

)




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class SystemSettingsAdvanced(Base):
    '''
    The advanced settings are a set of VMkernel options that specific configuration settings to be modified. These options are typically in place for specific workarounds or debugging.
    '''
    moid = 'ha-cli-handler-system-settings-advanced'
    def set(self, option, default=None, intvalue=None, stringvalue=None):
        '''
        Set the value of an advanced option.
        :param default: boolean, Reset the option to its default value.
        :param intvalue: long, If the option is an integer value use this option.
        :param option: string, The name of the option to set the value of. Example: "/Misc/HostName"
        :param stringvalue: string, If the option is a string use this option.
        :returns: void
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.system.settings.advanced.Set',
                            default=default,
                            intvalue=intvalue,
                            option=option,
                            stringvalue=stringvalue,
                            )
    def list(self, option=None, tree=None):
        '''
        List the advanced options available from the VMkernel.
        :param option: string, Only get the information for a single VMkernel advanced option.
        :param tree: string, Limit the list of advanced option to a specific sub tree.
        :returns: vim.EsxCLI.system.settings.advanced.list.SettingsAdvancedOption[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.system.settings.advanced.List',
                            option=option,
                            tree=tree,
                            )   



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def HostHyperThreadScheduleInfo(vim, *args, **kwargs):
    '''This data object type describes the CpuSchedulerSystem configuration for
    optimizing hyperthreading. The primary hyperthreading optimization employed by
    the CpuSchedulerSystem is to utilize hyperthreads as additional schedulable
    resources. Although hyperthreads provide limited additional concurrency,
    certain workloads (such as idling) can take advantage of these hyperthreads.
    This is particularly useful for SMP virtual machines that use gang scheduling.
    (Gang scheduling refers to a situation in which all of a parallel program's
    tasks are grouped into a "gang" and concurrently scheduled on distinct
    processors of a parallel computer system.)Changes to the hyperthreading
    optimization can take effect only after a system restart. Therefore, while it
    is possible to change the configuration at any time, the change will take
    effect only on the next boot.'''

    obj = vim.client.factory.create('{urn:vim25}HostHyperThreadScheduleInfo')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 3:
        raise IndexError('Expected at least 4 arguments got: %d' % len(args))

    required = [ 'active', 'available', 'config' ]
    optional = [ 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def VMwareVspanSession(vim, *args, **kwargs):
    '''This class defines the configuration of a Distributed Port Mirroring session. A
    Distributed Port Mirroring session is used to mirror network traffic
    (inbound/outbound) from a set of source ports to a set of destination ports, to
    assist in troubleshooting, and as input for security and other network analysis
    appliances.Once a port is configured to be a source transmitted port on any
    Distributed Port Mirroring session, it can not be used as the destination port
    in any Distributed Port Mirroring session. Overlapping source ports among
    different Distributed Port Mirroring session is permitted but overlapping
    destination ports among different Distributed Port Mirroring session is not
    permitted.'''

    obj = vim.client.factory.create('{urn:vim25}VMwareVspanSession')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 3:
        raise IndexError('Expected at least 4 arguments got: %d' % len(args))

    required = [ 'enabled', 'normalTrafficAllowed', 'stripOriginalVlan' ]
    optional = [ 'description', 'destinationPort', 'encapsulationVlanId', 'key',
        'mirroredPacketLength', 'name', 'sourcePortReceived', 'sourcePortTransmitted',
        'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def HostUnresolvedVmfsResolutionSpec(vim, *args, **kwargs):
    '''An unresolved VMFS volume is reported when one or more device partitions of
    volume are detected to have copies of extents of the volume. Such copies can be
    created via replication or snapshots, for example. This data object type
    describes how to resolve an unbound VMFS volume. The SCSI device path for each
    of the VMFS volume extent should be specified. For the current release, only
    head-extent needs to be specified. In future releases, we will allow user to
    specify explicitly all the extents which makes up a new Vmfs Volume.'''

    obj = vim.client.factory.create('{urn:vim25}HostUnresolvedVmfsResolutionSpec')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 2:
        raise IndexError('Expected at least 3 arguments got: %d' % len(args))

    required = [ 'extentDevicePath', 'uuidResolution' ]
    optional = [ 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj
